angular.module('chats.controller', [])

.controller('ChatsCtrl', function($scope, $_popup, $filter, $ionicActionSheet, $ionicPlatform, $cordovaVibration, $_account_cache,
    $_sdk, $_chat_list_cache, $_chat_list, $_notification, $state, $_options, $ionicScrollDelegate, $timeout, $rootScope, $_chat_detail_cache, $_helper, $_tracking, $_network, $_sound, $_location) {
    $scope.scrollPos;
    var refreshing = false;

    $scope.$on("$ionicView.beforeEnter", function(event, data){
        $_tracking.trackPage("Match Screen");
        $scope.login = {};
        $scope.login.loggedInUserId = $_sdk.getUserID();

        $scope.load = {};
        $scope.load.infinite = true;
        $scope.load.profile_image_alert = false; // Hinweis, ob Profilbild da ist oder nicht
        $scope.load.location_alert = false;

        var hasProfileImg = $_account_cache.getUserHasProfileImage();
        $scope.load.profile_image_alert = !hasProfileImg;

        var chat_list = $_chat_list_cache.getChatList();

        if(chat_list != null && chat_list != undefined && chat_list.length != undefined && chat_list.length > 0) {
            $scope.chats = chat_list;
            setUnreadMessages();
            getChatList();
        } else {
            getChatList();
        }

        if($scope.scrollPos != null && $scope.scrollPos != undefined) {
            $ionicScrollDelegate.scrollTo(0, $scope.scrollPos, false);
        }
    });

    $scope.$on("$ionicView.beforeLeave", function(){
        $scope.scrollPos = $ionicScrollDelegate.getScrollPosition().top;
    });

    $scope.$on('$ionicView.afterLeave', function(){
        $_helper.refreshNewTab();
    });

    $scope.$on('refresh-chat-list', function(event, args) { // Reagiert auf Push-Benachrichtigung in diesem Chat
        getChatList();
    });

    $scope.$on("connectivity-error", function(){
        $scope.load.infinite = false;
    });

    $scope.$on("connectivity-status-online", function(){
        $scope.load.infinite = true;
    });

    $scope.refresh = function() {
        if($scope.load.profile_image_alert == false && $scope.load.location_alert == false) {
            $_sound.playRefresh();
            refreshing = true;
            getChatList();
        }
    }

    function setUnreadMessages() {
        for(var i = 0; i <= $scope.chats.length-1; i++) {
            var chat_id = $scope.chats[i].match_id;

            var unread_messages = $_chat_detail_cache.getChatReadStatus(chat_id);
            
            if(unread_messages != null && unread_messages != undefined) {
                $scope.chats[i].unread_messages = unread_messages;
            }
        }
    }

    function getChatList() {
        $_chat_list.getChats().then(
            function(chats){
                $timeout(function(){
                    $scope.chats = chats;
                    $scope.load.profile_image_alert = false;

                    if(refreshing) {
                        refreshing = false;
                        $_sound.playRefreshFinish();
                    }

                    $scope.$broadcast('scroll.refreshComplete');
                    $scope.$broadcast('scroll.infiniteScrollComplete');
                }, 50);  
                $_helper.refreshBadges();
            },
            function(error){
                if(error.code == 46) { // Keine Ergebnisse
                    $scope.chats = {};
                    $scope.load.profile_image_alert = false;
                } else if(error.code == 265) {
                    $timeout(function(){
                        $scope.load.profile_image_alert = true;
                    }, 50);
                } else if(error.code == 271) {
                    $timeout(function(){
                        //$scope.load.location_alert = true;
                        var msg = $filter('translate')('GRANT_GPS_PERMISSIONS');
                        var type = $_notification.getNotificationTypes().ERROR;
                        $_notification.showNotificationBar(msg, type);
                    }, 50);
                } else {
                    $_notification.showNotificationBar(error.msg, error.type);
                    $scope.load.profile_image_alert = false;
                }

                if(error.code == 0 || error.code == -1) {
                    $scope.load.infinite = false;
                    $scope.load.profile_image_alert = false;
                }

                if(refreshing) {
                    refreshing = false;
                    $_sound.playRefreshFinish();
                }
                
                $scope.$broadcast('scroll.refreshComplete');
                $scope.$broadcast('scroll.infiniteScrollComplete');
            }
        );
    }

    $scope.loadMore = function() {
        if(($scope.chats == null || $scope.chats == undefined || $scope.chats.length == 0 || $scope.chats.length == undefined/* || $_chat_list.isOnFirstPage()*/)
            && $scope.load.profile_image_alert == false && $scope.load.location_alert == false) {
            //getChatList();

            // Wenn erste und letzte Seite
            if($_chat_list.isOnLastPage()) $scope.$broadcast('scroll.infiniteScrollComplete');
            
        } else if($_chat_list.isOnLastPage()) {
            $scope.$broadcast('scroll.infiniteScrollComplete');
        } else {
            if($scope.load.profile_image_alert == true || $scope.load.location_alert == true || $scope.chats.length < 6){
                $scope.$broadcast('scroll.infiniteScrollComplete');
                return;
            }
            $_chat_list.loadMore().then(
                function(res) {
                    $scope.$broadcast('scroll.infiniteScrollComplete');
                    if(angular.equals($scope.chats, res)) {
                        return;
                    }
                    
                    $timeout(function(){
                        var chatsArr = $scope.chats.concat(res);
                        var clearedChatsArr = removeDuplicates(chatsArr);

                        $scope.chats = clearedChatsArr;
                        $_chat_list_cache.saveChatList($scope.chats);
                        $ionicScrollDelegate.resize();
                    }, 50);
                },
                function(error) {
                    if(error.code == 0) { // Fehler im API Call
                        $scope.load.infinite = false;
                    }
                    $scope.$broadcast('scroll.infiniteScrollComplete');
                }
            );
        }
    }

    function removeDuplicates(chatsArr) {
        var duplicates = new Array();

        for(var i=0; i<=chatsArr.length-1; i++) {
            for(var j=0; j<=chatsArr.length-1; j++) {
                if(chatsArr[i].match_id == chatsArr[j].match_id && i != j) {
                    duplicates.push(i);
                    break;
                }
            }
        }

        for (var i = duplicates.length -1; i >= 0; i--) {
            chatsArr.splice(duplicates[i],1);
        }

        return chatsArr;
    }

    $scope.edit = function(item) {
        $_sound.playClick();
        var match_id = item.match_id;
        var user_id = item.user_id;

        getReasons();

        hideSheet = $ionicActionSheet.show({
            buttons: [
                { text: $filter('translate')('DELETE_CHAT') },
                { text: $filter('translate')('REPORT_USER') },
                { text: $filter('translate')('BLOCK_USER') }
            ],
            titleText: $filter('translate')('OPTIONS'),
            cancelText: $filter('translate')('CANCEL'),

            cancel: function() {
                $_sound.playClick();
            },
            buttonClicked: function(index) {
                hideSheet();
                if(index==0) $_popup.deleteChatPopup(match_id, false);
                else if (index==1) $_popup.reportUserPopup(user_id, $scope, false);
                else if (index==2) $_popup.blockUserPopup(user_id, false);
            }
        });
    }

    $scope.delete = function(item, index) {
        $_sound.playClick();
        var match_id = item.match_id;
        $_popup.deleteChatPopup(match_id, false, $scope, index);
    }

    function getReasons() {
        $_options.getReportReasons().then(
            function(list){
                $scope.reasons = list;
            },
            function(error){
                $_notification.showNotificationBar(error.msg, error.type);
            }
        );
    }

    $scope.showProfile = function(user_id) {
        var currentTab = $rootScope.rootCurrentTab;
        $state.go('tab.' + currentTab + '-profile', {id: user_id});
    }

    $scope.authorizeLocation = function() {
        $_location.requestLocationAuthorization().then(
            function(success){
                getChatList();
                return;
            },
            function(error){
                if(error == true) {
                    authorizeLocationPopup();
                }
            }
        );
    }

    function authorizeLocationPopup() {
        var text;

        if(cordova.platformId == 'ios') text = $filter('translate')('AUTHORIZE_LOCATION_TEXT_IOS');
        if(cordova.platformId == 'android') text = $filter('translate')('AUTHORIZE_LOCATION_TEXT_ANDROID');

        $_popup.alert(
            text,
            $filter('translate')('AUTHORIZE_LOCATION_HEAD')
        );
    }
});
