angular.module('flirt_detail_rate.controller', [])

.controller('FlirtDetailRateCtrl', function($scope, $_popup, $ionicActionSheet, $filter, $timeout, $_font_color_helper, $_flirt_detail_cache,
	$_flirt_detail, $stateParams, $_notification, $_flirt_rate, $ionicHistory, $state, $_flirts, $_tracking, $_helper, $_sound) {
	$scope.$on('$ionicView.beforeEnter', function(){
		var flirtId = $stateParams.flirtId;
		var roundId = $stateParams.roundId;

		if(flirtId == null || flirtId == undefined || flirtId == "" 
			|| roundId == null || roundId == undefined || roundId == "") {
			$ionicHistory.goBack();
		} else {
			$_tracking.trackPage("Rate Flirt");
			
			$_helper.showLoadingIndicator();
			getCachedData(flirtId, roundId);
			getFlirtDetail(flirtId, roundId);
		}
	});

	function getCachedData(flirtId, roundId) {
		$timeout(function() {
			$scope.flirt = $_flirt_detail_cache.getFlirtRound(flirtId, roundId);
			var flirtDetail = $_flirt_detail_cache.getFlirtDetail($stateParams.flirtId);
			$scope.flirt.profile_img = flirtDetail.other_user.profile_image;
			$scope.title = {};
			$scope.title.view_title = flirtDetail.other_user.name + ', ' + flirtDetail.other_user.age;
			$scope.title.sub_title = parseInt(flirtDetail.other_user.distance, 10) + ' km';
			$scope.title.is_verified = flirtDetail.other_user.is_verified;
		}, 25);
	}

	function getFlirtDetail(flirtId, roundId) {
		$_flirt_detail.getRoundDetail(flirtId, roundId).then(
			function(flirt){
				$timeout(function(){
					$scope.flirt = flirt;
					setAnswerFontSize();
				}, 25);

				$_helper.hideLoadingIndicator();
				$scope.$broadcast('scroll.refreshComplete');
			},
			function(error){
				$ionicHistory.goBack();
				$timeout(function(){
					$_notification.showNotificationBar(error.msg, error.type);

					$_helper.hideLoadingIndicator();
					$scope.$broadcast('scroll.refreshComplete');
				}, 250);
			}
		);
	}

	$scope.rejectAnswer = function() {
		$_sound.playZonk();
		$_helper.showSendIndicator();
		$_flirt_rate.rejectAnswer($stateParams.roundId).then(
			function(roundId){
				$_tracking.trackEvent("Flirt", "Rate", "Reject", roundId);
				$_helper.hideSendIndicator();
				$ionicHistory.goBack();
			},
			function(error){
				$_helper.hideSendIndicator();
				if(error.code == 141 || error.code == 142 || error.code == 162 || error.code == 160) {
					$ionicHistory.goBack();
				}
				if(error.code != 162 && error.code != 160) { // hat bereits rejected -> keine notification anzeigen
					$timeout(function(){
						$_notification.showNotificationBar(error.msg, error.type);
						$scope.$broadcast('scroll.refreshComplete');
					}, 250);
				}
			}
		);
	}

	$scope.acceptAnswer = function() {
		$_sound.playRatePositive();
		$_helper.showSendIndicator();
		$_flirt_rate.acceptAnswer($stateParams.roundId).then(
			function(roundId){
				$_tracking.trackEvent("Flirt", "Rate", "Accept", roundId);
				$_helper.hideSendIndicator();
				$ionicHistory.goBack();
			},
			function(error){
				$_helper.hideSendIndicator();
				if(error.code == 141 || error.code == 142 || error.code == 162 || error.code == 160) { // Flirt abgelaufen oder existiert nicht
					$ionicHistory.goBack();
				}
				if(error.code != 162 && error.code != 162) { // hat bereits accpeted -> keine notification anzeigen
					$timeout(function(){
						$_notification.showNotificationBar(error.msg, error.type);
						$scope.$broadcast('scroll.refreshComplete');
					}, 250);
				}
			}
		);
	}

	$scope.showOptionSheet = function() {
		$_sound.playClick();
		var flirtId = $stateParams.flirtId;
		var userId = $scope.flirt.user_id;

		$_flirts.showOptions(userId, flirtId, $scope, false);
  };

  $scope.showProfile = function() {
  	$_sound.playClick();
  }

  function setAnswerFontSize() {
  	$timeout(function () {
  		$scope.flirt.answer_text_font_size = $_font_color_helper.getFontSize($scope.flirt.answer.length);
  		$( '#answer-text' ).each(function () {
		    this.style.setProperty( 'font-size', $scope.flirt.answer_text_font_size, 'important' );
		});
  	}, 25);
  }
});