angular.module('flirt_list.controller', [])

.controller('FlirtsCtrl', function($scope, $_popup, $ionicActionSheet, $filter, $_flirt_list, $timeout, $_flirt_list_cache, $_sound,
	$_notification, $_options, $ionicScrollDelegate, $_flirts, $state, $rootScope, $ionicPlatform, $_helper, $_tracking, $_account_cache, $log, $_location) {
	var preventGoToFlirt = false;
	var refreshing = false;

	$scope.$on('$ionicView.beforeEnter', function(){
		$_tracking.trackPage("Flirts Screen");

		$scope.load = {};
        $scope.load.infinite = true;
        $scope.load.profile_image_alert = false; // Hinweis, ob Profilbild da ist oder nicht
        $scope.load.location_alert = false;
        $scope.countdownStarted = false;

        var hasProfileImg = $_account_cache.getUserHasProfileImage();
        $scope.load.profile_image_alert = !hasProfileImg;

		var flirts = $_flirt_list_cache.getFlirtList();

		if(flirts != null && flirts != undefined && flirts.length != undefined && flirts.length > 0) {
			$scope.flirts = flirts;
			if(!$scope.countdownStarted) $scope.countdown();
		} else {
			getFlirts();
		}
		preventGoToFlirt = false;
	});

	$scope.$on('$ionicView.beforeLeave', function(){
		$timeout.cancel(countdown);  // Countdown beenden.
		$rootScope.$broadcast('refresh-chat-list');
	});

	$scope.$on('$ionicView.afterLeave', function(){
		$_helper.refreshNewTab();
	});

	$scope.$on("connectivity-status-online", function(){
        $scope.load.infinite = true;
    });

	$ionicPlatform.on('pause', function() {
		$timeout.cancel(countdown);
		$scope.countdownStarted = false;
	});

	$ionicPlatform.on('resume', function() {
		if(!$scope.countdownStarted) $scope.countdown();
	});

	$scope.$on('refresh-flirt-list', function(event, args) { // Reagiert auf Push-Benachrichtigung in der Flirt-Liste
        getFlirts();
    });

    $scope.$on("connectivity-error", function(){
		$scope.load.infinite = false;
	});

	$scope.refresh = function(playSound) {
		playSound = (typeof playSound === 'undefined') ? true : playSound;
		if($scope.load.profile_image_alert == false && $scope.load.location_alert == false) {
			if(playSound) $_sound.playRefresh();
			refreshing = true;
			getFlirts();
		}
	}

	function getFlirts() {
		$_flirt_list.getFlirts().then(
			function(flirts){
				$timeout(function(){
					$scope.flirts = flirts;
					$log.debug("flirts");
					$log.debug($scope.flirts);
					$scope.load.profile_image_alert = false;
					if(!$scope.countdownStarted) $scope.countdown();

					if(refreshing) {
						refreshing = false;
						$_sound.playRefreshFinish();
					}

					$scope.$broadcast('scroll.refreshComplete');
					$scope.$broadcast('scroll.infiniteScrollComplete');
				}, 50);
				$_helper.refreshBadges();
			},
			function(error){
				if(error.code == 46) {
					$scope.flirts = {};
					$scope.load.profile_image_alert = false;
				} else if(error.code == 265) {
					$timeout(function(){
						$scope.load.profile_image_alert = true;
					}, 50);
				} else if(error.code == 271) {
					$timeout(function(){
						//$scope.load.location_alert = true;
						var msg = $filter('translate')('GRANT_GPS_PERMISSIONS');
						var type = $_notification.getNotificationTypes().ERROR;
						$_notification.showNotificationBar(msg, type);
					}, 50);
				} else {
					$_notification.showNotificationBar(error.msg, error.type);
					$scope.load.profile_image_alert = false;
				}

				if(error.code == 0 || error.code == -1) {
                    $scope.load.infinite = false;
                    $scope.load.profile_image_alert = false;
                }

                if(refreshing) {
                	refreshing = false;
                	$_sound.playRefreshFinish();
                }

				$scope.$broadcast('scroll.refreshComplete');
				$scope.$broadcast('scroll.infiniteScrollComplete');
			}
		);
	}

	$scope.loadMore = function() {
		//$log.debug("loadMore");
		//$log.debug($scope.flirts);
		//$log.debug($_flirt_list.isOnFirstPage());
		//$log.debug($_flirt_list.isOnLastPage());
		if(($scope.flirts == null || $scope.flirts == undefined || $scope.flirts.length == 0 || $scope.flirts.length == undefined/* || $_flirt_list.isOnFirstPage()*/)
			&& $scope.load.profile_image_alert == false && $scope.load.location_alert == false) {
			//getFlirts();

			// Wenn erste und letzte Seite
            if($_chat_list.isOnLastPage()) $scope.$broadcast('scroll.infiniteScrollComplete');

		} else if($_flirt_list.isOnLastPage()) {
			$scope.$broadcast('scroll.infiniteScrollComplete');
		} else {
			if($scope.load.profile_image_alert == true || $scope.load.location_alert == true || $scope.flirts.length < 6){
				$scope.$broadcast('scroll.infiniteScrollComplete');
				return;
			}

			$_flirt_list.loadMore().then(
				function(res) {
					$scope.$broadcast('scroll.infiniteScrollComplete');
                    if(angular.equals($scope.flirts, res)) {
                        return;
                    }

                    $timeout(function(){
                    	var flirtsArr = $scope.flirts.concat(res);
                    	var clearedFlirtsArr = removeDuplicates(flirtsArr);

                    	$scope.flirts = $_flirt_list.sortFlirtList(clearedFlirtsArr);
                    	$_flirt_list_cache.saveFlirtList($scope.flirts);
                    	$ionicScrollDelegate.resize();
                    }, 50);
				},
				function(error) {
					if(error.code == 0) { // Fehler im API Call
						$scope.load.infinite = false;
					}

					$scope.$broadcast('scroll.infiniteScrollComplete');
				}
			);
		}
	}

	function removeDuplicates(flirtArr) {
		var duplicates = new Array();

    	for(var i=0; i<=flirtArr.length-1; i++) {
    		for(var j=0; j<=flirtArr.length-1; j++) {
    			if(flirtArr[i].flirt_id == flirtArr[j].flirt_id && i != j) {
    				duplicates.push(i);
    				break;
    			}
    		}
    	}

    	for (var i = duplicates.length -1; i >= 0; i--) {
    		flirtArr.splice(duplicates[i],1);
    	}

    	return flirtArr;
	}

	$scope.goToFlirt = function(flirt_id) {
		if(!preventGoToFlirt) {
			$state.go('tab.' + $rootScope.rootCurrentTab + '-flirt-detail', {flirtId: flirt_id});
		}
	}

	$scope.edit = function(item, index) {
		$_sound.playClick();
		var userId = item.user_id;
		var flirtId = item.flirt_id;
		var goBack = false;

		$_flirts.showOptions(userId, flirtId, $scope, goBack, index);
	}

	$scope.delete = function(item, index) {
		preventGoToFlirt = true;
		$_sound.playClick();
		var flirtId = item.flirt_id;
		$_popup.deleteGamePopup(flirtId, false, $scope, index);
		$timeout(function(){
			preventGoToFlirt = false;
		}, 500);
	}

	$scope.authorizeLocation = function() {
		$_location.requestLocationAuthorization().then(
			function(success){
				getFlirts();
				return;
			},
			function(error){
				if(error == true) {
					authorizeLocationPopup();
				}
			}
		);
	}

	function authorizeLocationPopup() {
		var text;

		if(cordova.platformId == 'ios') text = $filter('translate')('AUTHORIZE_LOCATION_TEXT_IOS');
		if(cordova.platformId == 'android') text = $filter('translate')('AUTHORIZE_LOCATION_TEXT_ANDROID');

		$_popup.alert(
			text,
			$filter('translate')('AUTHORIZE_LOCATION_HEAD')
		);
	}

	var countdown = false;
	/* Lässt Countdown in der Liste laufen, wenn die Seite aufgerufen wird. */
	$scope.countdown = function() {
		$scope.countdownStarted = true;
		countdown = $timeout(function(){
			$log.debug("countdown");
			var now = moment.utc();
			var today = moment.utc(new Date());
			if($scope.flirts != null && $scope.flirts != undefined) {
				for(var i = 0; i <= $scope.flirts.length-1; i++) {
					if(!$scope.flirts[i].flirt_round_expired && $scope.flirts[i].my_round_status != "gameover" && $scope.flirts[i].my_round_status != "deleted") {
						var until = moment.utc($scope.flirts[i].flirt_round_valid_until);
						var count = today.countdown(until);

						if(moment() > until) {
							$scope.flirts[i].flirt_round_expired = true;
							$_flirt_list_cache.setFlirtAsExpired($scope.flirts[i].flirt_id);
						}

						var d = until.diff(today, 'days');
						var h = count.hours;
						var m = count.minutes;
						var s = count.seconds;

						//Setze Wort für den Tag
						var day_word = "";
						if(d > 1) {
							day_word = d + ' ' + $filter('translate')('DAYS');
						} else if(d == 1) {
							day_word = d + ' ' + $filter('translate')('DAY');
						}

						// Setzt 0 vor die Zahl, wenn einstellig
						h = $.trim(h).length === 1 ? '0' + h : h;
						m = $.trim(m).length === 1 ? '0' + m : m;
						s = $.trim(s).length === 1 ? '0' + s : s;

						$scope.flirts[i].time_left = day_word + ' ' + h + ':' + m + ':' + s;
					}
				}
			}
			

			$scope.countdown();
		}, 1000);
	};

	$scope.showOptionSheet = function() {
		$_sound.playClick();
		hideSheet = $ionicActionSheet.show({
			buttons: [
				{ text: $filter('translate')('DELETE_FAILED_FLIRTS') }
			],
			titleText: $filter('translate')('OPTIONS'),
			cancelText: $filter('translate')('CANCEL'),

			cancel: function() {
				$_sound.playClick();
			},
			buttonClicked: function(button_index) {
				hideSheet();
				if(button_index==0) $_popup.deleteFailedFlirtsPopup($scope);
			}
   		});
  };
});
