angular.module('account.service', [])
.factory('$_account', function($_sdk, $_helper, $_account_cache, $_notification, $filter, $_image_cache, $log, $_file_cache, $_location) {

    function createAccountObj(data) {
        return new Promise(function(resolve, reject) {
            var profile_img = $_helper.getMediumImg(data.profile_images);
            var verification_status = data.media_verification_status;
            var use_home_location = $_helper.isFalseOrTrue(data.use_home_location);
            var only_verified_profiles = $_helper.isFalseOrTrue(data.looking_for_only_verified);
            var allow_only_verified_flirts = $_helper.isFalseOrTrue(data.only_verified_flirts);
            if(verification_status == null || verification_status == undefined)
                verification_status = 'wait';

            d = new Date(data.birthdate + "T12:00:00.000Z"); // Fix für negative Zeitzonen

            var accountObj = {
                birthdate: d,
                datevalue: d,
                city: data.city,
                gender: data.gender,
                hashtag1: data.hashtag1,
                hashtag2: data.hashtag2,
                hashtag3: data.hashtag3,
                looking_for_age_from: data.looking_for_age_from,
                looking_for_age_to: data.looking_for_age_to,
                looking_for_distance: data.looking_for_distance,
                looking_for_gender: data.looking_for_gender,
                name: data.name,
                profile_img: profile_img,
                profile_img_id: data.object_has_media_media_id,
                email: data.email,
                login_type: data.login_type,
                current_media_object: data.current_media_object,
                verification_status: verification_status,
                only_verified_profiles: only_verified_profiles,
                allow_only_verified_flirts: allow_only_verified_flirts,
                use_home_location: use_home_location,
                home_location_city: data.home_location_city,
                home_location_id: data.home_location_id,
                current_points: data.current_points
            }

            resolve(accountObj);
        });
    }

    function changePassword(old_password, new_password) {
        return new Promise(function(resolve, reject){
            var callbacks = {
                onSuccessAck: function(successData) {
                    var msg = $filter('translate')('PASSWORD_CHANGE_SUCCESS');
                    var type = $_notification.getNotificationTypes().POSITIVE;

                    resolve($_helper.getObjForNotification(msg, type));
                },
                onFailAck: function(failData) {
                    var msg;
                    var type;
                    var notificationObj;

                    if(failData.code == 258) {
                        msg = $filter('translate')('OLD_PASSWORD_INCORRECT');
                        type = $_notification.getNotificationTypes().ERROR;
                    } else if(failData.code == 259) { // Altes Passwort == neues Passwort
                        msg = $filter('translate')('NEW_PASSWORD_NOT_SAME_LIKE_OLD');
                        type = $_notification.getNotificationTypes().ERROR;
                    } else {
                        msg = $filter('translate')('PASSWORD_CHANGE_FAILED');
                        type = $_notification.getNotificationTypes().ERROR;
                    }

                    notificationObj = $_helper.getObjForNotification(msg, type);
                    notificationObj.code = failData.code;

                    reject(notificationObj);
                },
                onValidator: function(validateData) {
                    var msg = $filter('translate')('FAILED_TO_LOAD_DATA');
                    var type = $_notification.getNotificationTypes().ERROR;

                    reject($_helper.getObjForNotification(msg, type));
                }
            }

            var params = {
                old_password: old_password,
                new_password: new_password
            };

            $_sdk.request("PUT", "/user/password", params, callbacks);
        });
    }

    function changeMailAddress(new_mail) {
        return new Promise(function(resolve, reject){
            var callbacks = {
                onSuccessAck: function(successData) {
                    var msg = $filter('translate')('MAIL_CHANGE_SUCCESS');
                    var type = $_notification.getNotificationTypes().POSITIVE;

                    resolve($_helper.getObjForNotification(msg, type));
                },
                onFailAck: function(failData) {
                    $log.debug(failData);
                    var msg;
                    var type;
                    var notificationObj;

                    if(failData.code == 255) { // Mail bereits vergeben
                        msg = $filter('translate')('MAIL_ALREADY_IN_USE');
                        type = $_notification.getNotificationTypes().ERROR;
                    } else if(failData.code == 47) {
                        msg = $filter('translate')('ALREADY_HAVE_REGISTERED');
                        type = $_notification.getNotificationTypes().ERROR;
                    } else {
                        msg = $filter('translate')('MAIL_CHANGE_FAILED');
                        type = $_notification.getNotificationTypes().ERROR;
                    }

                    notificationObj = $_helper.getObjForNotification(msg, type);
                    notificationObj.code = failData.code;

                    reject(notificationObj);
                },
                onValidator: function(validateData) {
                    var msg = $filter('translate')('MAIL_CHANGE_FAILED');
                    var type = $_notification.getNotificationTypes().ERROR;

                    reject($_helper.getObjForNotification(msg, type));
                }
            }

            var params = {
                new_email: new_mail
            };

            $_sdk.request("PUT", "/user/email", params, callbacks);
        });
    }

    function recoverPassword(mail) {
        return new Promise(function(resolve, reject){
            var callbacks = {
                onSuccessAck: function(successData) {
                    var msg = $filter('translate')('PLEASE_CHECK_YOUR_EMAIL');
                    var type = $_notification.getNotificationTypes().POSITIVE;

                    resolve($_helper.getObjForNotification(msg, type));
                },
                onFailAck: function(failData) {
                    $log.debug(failData);
                    var msg;
                    var type;
                    var notificationObj;

                    msg = $filter('translate')('FAILED_TO_RECOVER_PASSWORD');
                    type = $_notification.getNotificationTypes().ERROR;

                    notificationObj = $_helper.getObjForNotification(msg, type);
                    notificationObj.code = failData.code;

                    reject(notificationObj);
                },
                onValidator: function(validateData) {
                    var msg = $filter('translate')('FAILED_TO_RECOVER_PASSWORD');
                    var type = $_notification.getNotificationTypes().ERROR;

                    reject($_helper.getObjForNotification(msg, type));
                }
            }

            var params = {
                email: mail
            };

            $_sdk.request("POST", "/user/recover", params, callbacks);
        });
    }

    function getAccountData() {
      return new Promise(function(resolve, reject){
          var callbacks = {
              onSuccessAck: function(successData) {
                  createAccountObj(successData.data[0]).then(
                      function(accountData) {
                          var oldAccountImgId = $_account_cache.getOldAccountImageId();
                          var oldAccountImgIdBeforeUpload = $_account_cache.getOldAccountImageIdBeforeUpload();
                          var actAccountImgId = accountData.profile_img_id;
                          var userHasProfileImg = $_account_cache.getUserHasProfileImage();

                          $log.debug("IDS: " + oldAccountImgId + " - " + oldAccountImgIdBeforeUpload + " - " + actAccountImgId);


                          if(oldAccountImgId == 0 && oldAccountImgIdBeforeUpload == actAccountImgId && oldAccountImgIdBeforeUpload != undefined && oldAccountImgIdBeforeUpload != null) {
                            $log.debug("FROM CACHE");
                            // Load from Cache
                            $_file_cache.read($_account_cache.getAccountImage()).then(function(content) {
                              $log.debug("Erfolgreich Foto geladen");
                              accountData.profile_img = content;

                              resolve(accountData);
                            }).catch(function(err) {
                              $log.debug("Konnte Foto NICHT laden");
                              $log.debug(err);

                              $log.debug("FROM SERVER / CACHE FAIL");
                              // Load from Server
                              $_account_cache.saveOldAccountImageId(actAccountImgId);

                              $_account_cache.saveAccountData(accountData);
                              $_image_cache.cacheAllImages();

                              resolve(accountData);
                            });
                          } else if(actAccountImgId != null) {
                            $log.debug("FROM SERVER");
                            // Load from Server
                            $_account_cache.saveOldAccountImageId(actAccountImgId);

                            $_account_cache.saveAccountData(accountData);
                            $_image_cache.cacheAllImages();

                            resolve(accountData);
                          } else {
                            $log.debug("FROM PLACEHOLDER");
                            // Placeholder
                            accountData.profile_img = 'img/profile-img-default.jpg';

                            $_account_cache.saveAccountData(accountData);
                            $_image_cache.cacheAllImages();

                            resolve(accountData);
                          }

                          /*
                          // Bild im Cache == Bild auf Server?
                          if(oldAccountImgId != actAccountImgId && oldAccountImgId != 0 && oldAccountImgId != null) {
                            $_account_cache.saveOldAccountImageId(actAccountImgId);

                            $_account_cache.saveAccountData(accountData);
                            $_image_cache.cacheAllImages();

                            resolve(accountData);
                          } else if(oldAccountImgId == 0 && actAccountImgId != null) {
                            $_file_cache.read($_account_cache.getAccountImage()).then(function(content) {
                              $log.debug("Erfolgreich Foto geladen");
                              accountData.profile_img = content;

                              resolve(accountData);
                            }).catch(function(err) {
                              $log.debug("Konnte Foto NICHT laden");
                              $log.debug(err);
                            });
                          } else if(oldAccountImgId == actAccountImgId || oldAccountImgId == null) {
                            $_account_cache.saveOldAccountImageId(actAccountImgId);

                            $_account_cache.saveAccountData(accountData);
                            $_image_cache.cacheAllImages();

                            resolve(accountData);
                          } else {
                            accountData.profile_img = 'img/profile-img-default.jpg';

                            $_account_cache.saveAccountData(accountData);
                            $_image_cache.cacheAllImages();

                            resolve(accountData);
                          }
                          */
                      },
                      function(error) {
                          reject(error);
                      }
                  );
              },
              onFailAck: function(failData) {
                  var msg = $filter('translate')('REQUEST_FAILED');
                  var type = $_notification.getNotificationTypes().ERROR;

                  reject($_helper.getObjForNotification(msg, type));
              },
              onValidator: function(validateData) {
                  var msg = $filter('translate')('FAILED_TO_LOAD_DATA');
                  var type = $_notification.getNotificationTypes().ERROR;

                  reject($_helper.getObjForNotification(msg, type));
              }
          }

          $_sdk.request("GET", "/user/setting", null, callbacks);
      });
    }

    /* Speichert die eigenen Account-Einstellungen */
    function saveAccountData(data) {
          return new Promise(function(resolve, reject){
              var callbacks = {
                  onSuccessAck: function(successData) {
                      $log.debug("saveAccountData Test");
                      $log.debug(data);
                      data.profile_img = "";
                      $_account_cache.saveAccountData(data);

                      var msg = $filter('translate')('SAVED_SUCCESSFULLY_CHANGES');
                      var type = $_notification.getNotificationTypes().POSITIVE;

                      resolve($_helper.getObjForNotification(msg, type));
                  },
                  onFailAck: function(failData) {
                      $log.debug(failData);
                      var msg = $filter('translate')('REQUEST_FAILED');
                      var type = $_notification.getNotificationTypes().ERROR;

                      if(failData.code == 263)
                          msg = $filter('translate')('VERIFICATE_ACCOUNT_TO_USE_THIS_OPTIONS');

                      var notificationObj = $_helper.getObjForNotification(msg, type);
                      notificationObj.code = failData.code;

                      reject(notificationObj);
                  },
                  onValidator: function(validateData) {
                      var msg = $filter('translate')('FAILED_TO_SAVE_DATA');
                      var type = $_notification.getNotificationTypes().ERROR;

                      reject($_helper.getObjForNotification(msg, type));
                  }
              }

              data.birthdate = moment(data.birthdate).format('YYYY-MM-DD');
              data.only_verified_flirts = $_helper.getFalseOrTrueFlag(data.allow_only_verified_flirts);
              data.looking_for_only_verified = $_helper.getFalseOrTrueFlag(data.only_verified_profiles);
              data.use_home_location = $_helper.getFalseOrTrueFlag(data.use_home_location);

              $_sdk.request("PUT", "/user/setting", data, callbacks);
          });
    }

    function getShareLink() {
        return new Promise(function(resolve, reject){
            var callbacks = {
                onSuccessAck: function(successData) {
                    resolve(successData);
                },
                onFailAck: function(failData) {
                    $log.debug(failData);
                    var msg = $filter('translate')('SHARE_FAILED');
                    var type = $_notification.getNotificationTypes().ERROR;

                    reject($_helper.getObjForNotification(msg, type));
                },
                onValidator: function(validateData) {
                    var msg = $filter('translate')('SHARE_FAILED');
                    var type = $_notification.getNotificationTypes().ERROR;

                    reject($_helper.getObjForNotification(msg, type));
                }
            }

            $_sdk.request("GET", "/sharelink", null, callbacks);
        });
    }

    var setHomeLocation = function(resolve, reject){
      return new Promise(function(resolve, reject){
        var callbacks = {
            onSuccessAck: function(successData) {
                resolve(successData);
            },
            onFailAck: function(failData) {
                $log.debug(failData);
                var msg = $filter('translate')('REQUEST_FAILED');
                var type = $_notification.getNotificationTypes().ERROR;

                reject($_helper.getObjForNotification(msg, type));
            },
            onValidator: function(validateData) {
                var msg = $filter('translate')('REQUEST_FAILED');
                var type = $_notification.getNotificationTypes().ERROR;

                reject($_helper.getObjForNotification(msg, type));
            }
        }

        // Hole erst die aktuelle Location des Benutzers
        $_location.getCurrentLocation().then(
            function(locationObj) {
                $log.debug("Location success");
                $log.debug(locationObj);
                var params = {
                    lat: locationObj.lat,
                    lng: locationObj.lng
                }

                $_sdk.request("POST", "/user/homelocation", params, callbacks);
                    
            },
            function(error) {
                $log.debug("Location error");
                $log.debug(error);

                var msg = $filter('translate')('COULD_NOT_READ_GPS_COORDINATES');
                var type = $_notification.getNotificationTypes().ERROR;

                reject($_helper.getObjForNotification(msg, type));
            }
        );
      });
    }

    return {
        getAccountData: function() {
          return getAccountData();
        },
        saveAccountData: function(data) {
          return saveAccountData(data);
        },
        changePassword: function(old_password, new_password) {
            return changePassword(old_password, new_password);
        },
        changeMailAddress: function(new_mail) {
            return changeMailAddress(new_mail);
        },
        recoverPassword: function(mail) {
            return recoverPassword(mail);
        },
        getShareLink: function() {
            return getShareLink();
        },
        setHomeLocation: function() {
            return setHomeLocation();
        }
    }
});
