angular.module('user_block.service', [])
.factory('$_user_block', function($_sdk, $_helper, $_flirt_list_cache, $_notification, $filter, $_image_cache, $_blocked_users_cache, $log) {
    var pageURL = "#";
    var onFirstPage = true;
    var onLastPage = false;

    /* Holt die Liste der geblockten Nutzer von der API */
    var getPromise = function(resolve, reject){
        var callbacks = {
            onSuccessAck: function(successData) {
                createBlockedUsersList(successData).then(function(data){
                    $_blocked_users_cache.saveBlockedUsers(data);
                    getNextPageURL(successData);
                    resolve(data);
                }, function(error){
                    reject(error);
                })
            },
            onFailAck: function(failData) {
                $log.debug(failData);
                var msg = $filter('translate')('FAILED_TO_LOAD_DATA');
                var type = $_notification.getNotificationTypes().ERROR;
                var notificationObj = $_helper.getObjForNotification(msg, type);

                notificationObj.code = failData.code;

                reject(notificationObj);
            },
            onValidator: function(validateData) {
                var msg = $filter('translate')('FAILED_TO_LOAD_DATA');
                var type = $_notification.getNotificationTypes().ERROR;

                reject($_helper.getObjForNotification(msg, type));
            }
        }

        if(onLastPage) {
            var msg = $filter('translate')('NO_MORE_DATA_TO_LOAD');
            var type = $_notification.getNotificationTypes().ERROR;

            reject($_helper.getObjForNotification(msg, type));
        } else {
            if(!onFirstPage) {
                $_sdk.request("GET", pageURL, null, callbacks);
            } else {
                $_sdk.request("GET", "/block", null, callbacks);
            }
        }
    }

    /* Erstellt die Liste der geblockten Nutzer */
    function createBlockedUsersList(data) {
        return new Promise(function(resolve, reject) {
            var users = [];
            var blockedListPusher = function(item, index) {
                try {
                    var profile_img = $_helper.getSmallImg(item.profile_images);
                    var block_updated_at = moment(item.block_updated_at).format("DD.MM.YYYY");

                    var userObj = {
                        block_id: item.block_id,
                        user_id: item.user_id,
                        profile_img: profile_img,
                        name: item.block_user_name,
                        block_date: block_updated_at
                    }

                    users.push(userObj);

                    if(data.from + index == data.to || index == data.to - 1 || index == data.per_page-1) {
                        $_image_cache.cacheAllImages();
                        resolve(users);
                    } else if(index > data.total) {
                        var msg = $filter('translate')('OOPS_SOMETHING_WENT_WRONG');
                        var type = $_notification.getNotificationTypes().ERROR;

                        reject($_helper.getObjForNotification(msg, type));
                    }

                } catch(e) {
                    var msg = $filter('translate')('OOPS_SOMETHING_WENT_WRONG');
                    var type = $_notification.getNotificationTypes().ERROR;
                    var notificationObj = $_helper.getObjForNotification(msg, type);

                    notificationObj.code = 0;
                    reject(notificationObj);
                }
            };

            $_sdk.iterator(data, blockedListPusher);
        });
    }

    /* Setzt die URL der nächsten Seite, falls vorhanden.  */
    function getNextPageURL(successData) {
        $_sdk.hasNextPageURL(successData, function(nextPageURL) {
            pageURL = nextPageURL;
            onFirstPage = false;
            onLastPage = false;
        }, function() {
            pageURL = "#";
            onFirstPage = false;
            onLastPage = true;
        });
    }

    return {
        getBlockedUsers: function() {
            pageURL = "#";
            onFirstPage = true;
            onLastPage = false;
            return new Promise(getPromise);
        },
        loadMore: function() {
            return new Promise(getPromise);
        },
        isOnLastPage: function() {
            return onLastPage;
        },
        isOnFirstPage: function() {
            return onFirstPage;
        }
    }
});
