angular.module('blury.service', [])
.factory('$_blury', function($_sdk, $_users_cache, $_helper, $filter, $_notification, $_location, $_flirt_detail_cache, $_image_cache, $_account_cache, $log) {
    var pageURL = "#";
    var onFirstPage = true;
    var onLastPage = false;

    // Führt den GET Call aus
    var getPromise = function(resolve, reject){
        var callbacks = {
            onSuccessAck: function(successData) {
                $_account_cache.setUserHasProfileImage(true);
                getUsersObj(successData).then(
                    function(usersObj) {
                        cacheUsers(usersObj);
                        getNextPageURL(successData);
                        resolve(usersObj);
                    },
                    function(error) {
                        reject(error);
                    }
                );
            },
            onFailAck: function(failData) {
                $log.debug(failData);
                $_account_cache.setUserHasProfileImage(true);
                if(failData.code == 46) {
                    var msg = $filter('translate')('NO_RESULTS_CHECK_SETTINGS');
                    var type = $_notification.getNotificationTypes().ERROR;
                    var notification = $_helper.getObjForNotification(msg, type);

                    notification.code = 46;
                    cacheUsers({});
                    onLastPage = true;

                    reject(notification);
                } else if(failData.code == 265) {
                    $_account_cache.setUserHasProfileImage(false); // nur an dieser Stelle auf false setzen, sonst true.

                    reject(failData);
                } else if(failData.code == 271) {
                    reject(failData);
                } else {
                    var msg = $filter('translate')('REQUEST_FAILED');
                    var type = $_notification.getNotificationTypes().ERROR;

                    reject($_helper.getObjForNotification(msg, type));
                }
            },
            onValidator: function(validateData) {
                $_account_cache.setUserHasProfileImage(true);
                var msg = $filter('translate')('FAILED_TO_LOAD_DATA');
                var type = $_notification.getNotificationTypes().ERROR;

                reject($_helper.getObjForNotification(msg, type));
            }
        }

        // Hole erst die aktuelle Location des Benutzers
        $_location.getCurrentLocation().then(
            function(locationObj) {
                $log.debug("Location success");
                $log.debug(locationObj);
                var params = {
                    lat: locationObj.lat,
                    lng: locationObj.lng
                }

                if(onLastPage) {
                    var msg = $filter('translate')('NO_MORE_DATA_TO_LOAD');
                    var type = $_notification.getNotificationTypes().ERROR;

                    reject($_helper.getObjForNotification(msg, type));
                } else {
                    if(!onFirstPage) {
                        $_sdk.request("GET", pageURL, params, callbacks);
                    } else {
                        $_sdk.request("GET", "/users", params, callbacks);
                    }
                }
            },
            function(error) {
                $log.debug("Location error");
                $log.debug(error);
                if(error.code == -1 && (error.errorType == 'GRANT_GPS_PERMISSIONS' || error.errorType == 'GPS_NOT_ENABLED')) {
                    reject(error); // GPS Problem "Could not read GPS position" / "GPS not enabled"
                } else if(onLastPage) {
                    var msg = $filter('translate')('NO_MORE_DATA_TO_LOAD');
                    var type = $_notification.getNotificationTypes().ERROR;

                    reject($_helper.getObjForNotification(msg, type));
                } else {
                    if(!onFirstPage) {
                        $_sdk.request("GET", pageURL, null, callbacks);
                    } else {
                        $_sdk.request("GET", "/users", null, callbacks);
                    }
                }
            }
        )
    }

    // Erstellt eine Liste mit Users Objekten
	function getUsersObj(data) {
		return new Promise(function(resolve, reject){
			var users = [];

            var usersPusher = function(item, index) {
                try {
                    var can_start_flirt = $_helper.isFalseOrTrue(item.can_start_flirt);
                    var profile_img = $_helper.getSmallImg(item.profile_images);

                    if(item.profile_images == null || item.profile_images == undefined || item.profile_images.length < 1 || item.profile_images.small == null
                        || item.profile_images.small == undefined || profile_img == null) {
                        if(data.from + index == data.to || index == data.to - 1 || index == data.per_page-1){
                            resolve(users);
                            return;
                        }
                        else
                            return;
                    }

                    var distance = parseInt(item.distance, 10);
                    var is_verified = $_helper.isFalseOrTrue(item.media_verification);

                    var user = {
                        user_id: item.user_id,
                        age: item.age,
                        profile_img: profile_img,
                        distance: distance,
                        use_home_location: item.use_home_location,
                        lat: item.lat,
                        lng: item.lng,
                        is_verified: is_verified,
                        can_start_flirt: can_start_flirt,
                        //last_activity: (item.last_activity),
                        online_until: (item.online_until),
                        recently_online_until: (item.recently_online_until)
                    };

                    users.push(user);
                    if(data.from + index == data.to || index == data.to - 1 || index == data.per_page-1) {
                        $_image_cache.cacheAllImages();
                        resolve(users);
                    } else if(index > data.total) {
                        var msg = $filter('translate')('OOPS_SOMETHING_WENT_WRONG');
                        var type = $_notification.getNotificationTypes().ERROR;

                        reject($_helper.getObjForNotification(msg, type));
                    }

                } catch(e) {
                    var msg = $filter('translate')('OOPS_SOMETHING_WENT_WRONG');
                    var type = $_notification.getNotificationTypes().ERROR;

                    var notificationObj = $_helper.getObjForNotification(msg, type);

                    notificationObj.code = 0;

                    reject(notificationObj);
                }
            };

            $_sdk.iterator(data, usersPusher);
		});
	}

    // Holt sich die Nächste page URL
    function getNextPageURL(successData) {
        $_sdk.hasNextPageURL(successData, function(nextPageURL) {
            pageURL = nextPageURL;
            onFirstPage = false;
            onLastPage = false;
        }, function() {
            pageURL = "#";
            onFirstPage = false;
            onLastPage = true;
        });
    }

    // Sichert die User Liste in den Cache
    function cacheUsers(usersObj) {
        if(onFirstPage) {
            $_users_cache.saveUsers(usersObj);
        } else {
            var cachedObj = $_users_cache.getUsers();
            var newObj = cachedObj.concat(usersObj);
            $_users_cache.saveUsers(newObj);
        }
    }

	return {
		getUsers() {
            pageURL = "#";
            onFirstPage = true;
            onLastPage = false;
			return new Promise(getPromise);
		},
        loadMore() {
            return new Promise(getPromise);
        },
        startFlirt(id) {
            return new Promise(function(resolve, reject){
                var callbacks = {
                    onSuccessAck: function(successData) {
                        resolve(successData.flirt_id);
                    },
                    onFailAck: function(failData) {
                        if(failData.code == 240) {
                            var flirt_id = failData.flirt_id;
                            resolve(flirt_id);
                        } else if(failData.code == 133) {
                            var msg = $filter('translate')('USER_DOES_NOT_EXIST');
                            var type = $_notification.getNotificationTypes().ERROR;
                            reject({ msg: msg, type: type, code: failData.code });
                        } else {
                            var msg = $filter('translate')('REQUEST_FAILED');
                            var type = $_notification.getNotificationTypes().ERROR;
                            reject({ msg: msg, type: type, code: failData.code });
                        }
                    },
                    onValidator: function(validateData) {
                        var msg = $filter('translate')('FAILED_TO_LOAD_DATA');
                        var type = $_notification.getNotificationTypes().ERROR;

                        reject({ msg: msg, type: type, code: -1 });
                    }
                }

                var params = {
                    match_user_id: id
                }

                $_sdk.request("POST", "/flirt", params, callbacks);
            });
        },
        isOnLastPage() {
            return onLastPage;
        },
        isOnFirstPage() {
            return onFirstPage;
        },
        removeDuplicates(originalArray, prop) {
            var trimmedArray = [];
            var values = [];
            var value;

            for(var i = 0; i < originalArray.length; i++) {
                value = originalArray[i][prop];

                if(values.indexOf(value) === -1) {
                  trimmedArray.push(originalArray[i]);
                  values.push(value);
                }
            }

            return trimmedArray;
        },
        removeUserFromList(list, id) {
            for(var i = 0; i < list.length; i++) {
                var obj = list[i];

                if(obj.user_id == id) {
                    list.splice(i, 1);
                    i--;
                }
            }

            return list;
        }
	};
});
