angular.module('chat_detail_cache.service', [])
.factory('$_chat_detail_cache', function(CacheFactory, CACHE_ROUTE, $log) {
	const LIMIT_FOR_FAILED_MESSAGES = 30;

	if (!CacheFactory.get(CACHE_ROUTE().CHAT_DETAIL_CACHE)) {
		CacheFactory.createCache(CACHE_ROUTE().CHAT_DETAIL_CACHE, {
			storageMode: 'localStorage'
		});
	}

	var chatDetailCache = CacheFactory.get(CACHE_ROUTE().CHAT_DETAIL_CACHE);

	return {
		saveChat: function(id, obj) {
			chatDetailCache.put(CACHE_ROUTE().CHAT_DETAIL + '' + id, obj);
		},
		getChat: function(id) {
			var chat = chatDetailCache.get(CACHE_ROUTE().CHAT_DETAIL + '' + id);
			return chat;
		},
		markChatAsRead: function(id) {
			chatDetailCache.put(CACHE_ROUTE().READ_CHAT_MESSAGES + '' + id, 0);
		},
		markChatAsUnRead: function(id, unread_messages) {
			chatDetailCache.put(CACHE_ROUTE().READ_CHAT_MESSAGES + '' + id, unread_messages);
		},
		getChatReadStatus: function(id) {
			return chatDetailCache.get(CACHE_ROUTE().READ_CHAT_MESSAGES + '' + id);
		},
		putUnsendMessage: function(obj, chatId) {
			var messageArr = this.getUnsendMessages(chatId);

			if(messageArr == null || messageArr == undefined) {
				messageArr = [];
			}
			messageArr = messageArr.concat(obj);
			chatDetailCache.put(CACHE_ROUTE().UNSEND_MESSAGES + '' + chatId, messageArr);
		},
		putNewUnsendMessageArr: function(chatId, messageArr) {
			chatDetailCache.put(CACHE_ROUTE().UNSEND_MESSAGES + '' + chatId, messageArr);
		},
		getUnsendMessages: function(chatId) {
			var unsendMessagesArr = chatDetailCache.get(CACHE_ROUTE().UNSEND_MESSAGES + '' + chatId);

			// Überprüft ob Nachrichten älter als zwei Minuten sind, falls ja -> als fehlgeschlagen markiert.
			if(unsendMessagesArr != null && unsendMessagesArr != undefined) {
				for(var i=0; i<=unsendMessagesArr.length-1; i++) {
					unsendMessageObj = unsendMessagesArr[i];

					var sent_date = moment(unsendMessageObj.sorting_date);
					var now = moment();

					if(now.diff(sent_date, 'seconds') > LIMIT_FOR_FAILED_MESSAGES) {
						unsendMessageObj.status = "failed";
					}
				}
			}

			return unsendMessagesArr;
		},
		removeUnsendMessage: function(chatId, msgId) {
			var messageArr = this.getUnsendMessages(chatId);

			if(messageArr != null && messageArr != undefined) {
				for(var i=0; i<=messageArr.length-1; i++) {
					if(messageArr[i].id == msgId) {
						$log.debug("removeUnsendMessage: "+msgId);
						messageArr.splice(i, 1);
						this.putNewUnsendMessageArr(chatId, messageArr);
						break;
					}
				}
			}
		},
		putTypedMessage: function(chatId, msg) { // Speichert eingegebene, nicht abgeschickte Nachrichten
			chatDetailCache.put(CACHE_ROUTE().CHAT_TYPED_MESSAGE + '' + chatId, msg);
		},
		getCachedMessage: function(chatId) { // Holt eingegebene, nicht abgeschickte Nachrichten
			return chatDetailCache.get(CACHE_ROUTE().CHAT_TYPED_MESSAGE + '' + chatId);
		}
	}
});