angular.module('file_cache.service', [])

  .factory('$_file_cache', function($_sdk, $_helper, $window, $log) {

    var type = LocalFileSystem.PERSISTENT;
    var size = 20 * 1024 * 1024; // 20 MB
    var prefix = "blury_cache_file";
    var extension = ".txt";

    var contentChecker = function(content) {

        return new Promise(function(resolve, reject) {

          if(typeof content === "undefined") {
            reject("$_file_cache: Content is not valid (undefined). Should be an object or an string");
          } else if(typeof content === "object") {
            resolve(JSON.stringify(content));
          } else if(typeof content === "string") {
            resolve("" + content);
          } else {
            reject("$_file_cache: Content is not valid (unknown error). Should be an object or an string");
          }

        });

    };

    return {

      save: function(fileName, content) {

        var fileCache = this;

        fileCache.remove(prefix + fileName + extension).then(function() {
          $log.debug("$_file_cache.save: file could be deleted");
        }).catch(function(err) {
          $log.debug("$_file_cache.save: file could not be deleted");
        });

        return new Promise(function(resolve, reject) {
          var resultContent = null;
          contentChecker(content).then(function(content) {
            resultContent = content;
          }).catch(function(err) {
            reject(err);
          });

          $window.requestFileSystem(type, size, gotFS, fail);

          function gotFS(fileSystem) {
              $log.debug("$_file_cache::gotFS");
              fileSystem.root.getFile(prefix + fileName + extension, {create: true, exclusive: false}, gotFileEntry, fail);
          }

          function gotFileEntry(fileEntry) {
              $log.debug("$_file_cache::gotFileEntry");
              fileEntry.createWriter(gotFileWriter, fail);
          }

          function gotFileWriter(writer) {
            $log.debug("$_file_cache::gotFileWriter");
            writer.onwriteend = function(evt) {
              $log.debug("$_file_cache::gotFileWriter onwriteend");
            };
            writer.write(resultContent);
            resolve();
          }

          function fail(error) {
              reject("$_file_cache: Cannot save cache file. Error-Code: " + error.code);
          }

        });

      },

      read: function(fileName) {

        return new Promise(function(resolve, reject) {

          $log.debug("fileCacheService::READ");

          $window.requestFileSystem(type, size, successCallback, fail);

          function successCallback(fs) {

            fs.root.getFile(prefix + fileName + extension, {}, function(fileEntry) {
               fileEntry.file(function(file) {
                  var reader = new FileReader();
                  reader.onloadend = function(e) {
                    var readContent = this.result;
                    contentChecker(readContent).then(function(getReadContent) {
                      resolve(getReadContent);
                    }).catch(function(err) {
                      reject(err);
                    });
                  };
                  reader.readAsText(file);
               }, fail);

            }, fail);

          }

          function fail(error) {
            reject("$_file_cache: Cannot read cache file. Error-Code: " + error.code);
          }

        });

      },

      remove: function(fileName) {

        return new Promise(function(resolve, reject) {

          $window.requestFileSystem(type, size, successCallback, fail);

          function successCallback(fs) {

            fs.root.getFile(prefix + fileName + extension, {}, function(fileEntry) {

              fileEntry.remove(function(){
                resolve();
              }, fail, function(){
                fail(99999999); // Wird noch optimiert
              });

            }, fail);

          }

          function fail(error) {
            reject("$_file_cache: Cannot delete file. Error-Code: " + error.code);
          }

        });

      }

    };

  });