angular.module('flirt_detail_cache.service', [])
.factory('$_flirt_detail_cache', function(CacheFactory, CACHE_ROUTE) {
	if (!CacheFactory.get(CACHE_ROUTE().FLIRT_DETAIL_CACHE)) {
		CacheFactory.createCache(CACHE_ROUTE().FLIRT_DETAIL_CACHE, {
			maxAge: 120 * 60 * 60 * 1000,
			deleteOnExpire: 'aggressive',
			recycleFreq: 120 * 60 * 60 * 1000,
			storageMode: 'localStorage'
		});
	}

	// Speichert die Foto-URL der jeweiligen Flirt Rounds
	if (!CacheFactory.get(CACHE_ROUTE().FLIRT_ROUND_IMAGE_CACHE)) {
		CacheFactory.createCache(CACHE_ROUTE().FLIRT_ROUND_IMAGE_CACHE, {
			maxAge: 120 * 60 * 60 * 1000,
			deleteOnExpire: 'aggressive',
			recycleFreq: 240 * 60 * 60 * 1000,
			storageMode: 'localStorage'
		});
	}

	var flirtDetailCache = CacheFactory.get(CACHE_ROUTE().FLIRT_DETAIL_CACHE);
	var flirtRoundImageCache = CacheFactory.get(CACHE_ROUTE().FLIRT_ROUND_IMAGE_CACHE);

	return {
		saveFlirtDetail: function(obj, id) {
			flirtDetailCache.put(CACHE_ROUTE().FLIRT_DETAIL + '' + id, obj);
		},
		getFlirtDetail: function(id) {
			var flirt_detail = flirtDetailCache.get(CACHE_ROUTE().FLIRT_DETAIL + '' + id);
			return flirt_detail;
		},
		removeFlirtDetail: function(id) {
			flirtDetailCache.remove(CACHE_ROUTE().FLIRT_DETAIL + '' + id);
		},
		saveFlirtRounds: function(obj, flirtId) {
			for(var i=0; i<=obj.length-1; i++) {
				var round = obj[i];
				var roundObj = {
					my_round_status: round.my_round_status,
					question: round.question,
					round_id: round.round_id,
					user_id: round.user_id
				}

				flirtDetailCache.put(CACHE_ROUTE().FLIRT_ROUND + '' + flirtId + '/' + round.round_id, roundObj);
			}
		},
		getFlirtRound: function(flirtId, roundId) {
			return flirtDetailCache.get(CACHE_ROUTE().FLIRT_ROUND + '' + flirtId + '/' + roundId);
		},
		getFlirtRoundImage: function(flirtId) { //  Speichert das Foto zur jeweiligen Runde im Flirt (um es z.B. beim Entblurren zu benutzen)
			return flirtRoundImageCache.get(CACHE_ROUTE().FLIRT_ROUND_IMAGE + '' + flirtId);
		},
		saveFlirtRoundImage: function(flirtId, round, url) {
			var obj = this.getFlirtRoundImage(flirtId);

			if(obj == null || obj == undefined) {
				obj = {};
			}

			obj[round] = url;

			flirtRoundImageCache.put(CACHE_ROUTE().FLIRT_ROUND_IMAGE + '' + flirtId, obj);
		},
		saveStartedFlirtRound: function(userId) {
			flirtDetailCache.put(CACHE_ROUTE().STARTED_FLIRT_WITH_USER + '' + userId, true);
		},
		getStartedFlirtRound: function(userId) {
			return flirtDetailCache.get(CACHE_ROUTE().STARTED_FLIRT_WITH_USER + '' + userId);
		},
		deleteStartedFlirtRound: function(userId) {
			flirtDetailCache.put(CACHE_ROUTE().STARTED_FLIRT_WITH_USER + '' + userId, false);
		},
		deleteAllStartedFlirtRounds: function() {
			flirtDetailCache.removeAll();
		}
	}
});