angular.module('flirt_list_cache.service', [])
.factory('$_flirt_list_cache', function(CacheFactory, CACHE_ROUTE) {
	if (!CacheFactory.get(CACHE_ROUTE().FLIRT_LIST_CACHE)) {
		CacheFactory.createCache(CACHE_ROUTE().FLIRT_LIST_CACHE, {
			maxAge: 120 * 60 * 60 * 1000,
			deleteOnExpire: 'aggressive',
			recycleFreq: 240 * 60 * 60 * 1000,
			storageMode: 'localStorage'
		});
	}

	var flirtListCache = CacheFactory.get(CACHE_ROUTE().FLIRT_LIST_CACHE);

	return {
		saveFlirtList: function(obj) {
			flirtListCache.put(CACHE_ROUTE().FLIRT_LIST, obj);
		},
		getFlirtList: function() {
			var flirt_list = flirtListCache.get(CACHE_ROUTE().FLIRT_LIST);
			if(flirt_list == null || flirt_list == undefined) {
				return null;
			}
			return flirt_list;
		},
		deleteElementFromList: function(flirt_id) {
			var flirt_list = this.getFlirtList();

			if(flirt_list != null && flirt_list != undefined) {
				for(var i=0; i<=flirt_list.length-1; i++) {
					if(flirt_list[i].flirt_id == flirt_id) {
						flirt_list.splice(i, 1);
					}
				}
				this.saveFlirtList(flirt_list);
			}
			
		},
		updateFlirtRoundStatus: function(flirt_id, flirt_state, round, round_valid_until, is_expired) { // Updatet den Flirt-Round Status in der Liste, die im Cache gespeichert ist
			var flirt_list = this.getFlirtList();
			if(flirt_list != null && flirt_list != undefined) {
				for(var i=0; i<=flirt_list.length-1; i++) {
					if(flirt_list[i].flirt_id == flirt_id) {
						var flirtObj = flirt_list[i];

						flirtObj.my_round_status = flirt_state;
						flirtObj.round = round;
						flirtObj.flirt_round_valid_until = round_valid_until;
						flirtObj.flirt_round_expired = is_expired;

						flirt_list[i] = flirtObj;
					}
				}
				this.saveFlirtList(flirt_list);
			}
		},
		setFlirtAsExpired: function(flirt_id) {
			var flirt_list = this.getFlirtList();
			if(flirt_list != null && flirt_list != undefined) {
				for(var i=0; i<=flirt_list.length-1; i++) {
					if(flirt_list[i].flirt_id == flirt_id) {
						var flirtObj = flirt_list[i];

						flirtObj.flirt_round_expired = true;
						flirt_list[i] = flirtObj;
					}
				}
			}
			this.saveFlirtList(flirt_list);
		}
	}
});