angular.module('image_cache.service', [])
.factory('$_image_cache', function(CacheFactory, CACHE_ROUTE, $rootScope, $_network, $log) {
	if (!CacheFactory.get(CACHE_ROUTE().IMAGE_TIMESTAMP_CACHE)) {
		CacheFactory.createCache(CACHE_ROUTE().IMAGE_TIMESTAMP_CACHE, {
			storageMode: 'localStorage'
		});
	}

	var maxCacheTimeSec = 2 * 24 * 60 * 60 * 1000; // 2 Tage
	var timestampCache = CacheFactory.get(CACHE_ROUTE().IMAGE_TIMESTAMP_CACHE);

	function b64toBlob(b64Data, contentType, sliceSize) {
        contentType = contentType || '';
        sliceSize = sliceSize || 512;

        var byteCharacters = atob(b64Data);
        var byteArrays = [];

        for (var offset = 0; offset < byteCharacters.length; offset += sliceSize) {
            var slice = byteCharacters.slice(offset, offset + sliceSize);

            var byteNumbers = new Array(slice.length);
            for (var i = 0; i < slice.length; i++) {
                byteNumbers[i] = slice.charCodeAt(i);
            }

            var byteArray = new Uint8Array(byteNumbers);

            byteArrays.push(byteArray);
        }

      var blob = new Blob(byteArrays, {type: contentType});
      return blob;
  	}

  	function savebase64AsImageFile(folderpath,filename,content,contentType){ // Speichert base64 Bild in den angegebenen Pfad. 
  		return new Promise(function(resolve, reject){
  			var DataBlob = b64toBlob(content,contentType);
    
		    $log.debug("Starting to write the file :");
		    
		    window.resolveLocalFileSystemURL(folderpath, function(dir) {
		        $log.debug("Access to the directory granted succesfully");
				dir.getFile(filename, {create:true}, function(file) {
		            $log.debug("File created succesfully.");
		            file.createWriter(function(fileWriter) {
		                $log.debug("Writing content to file");
		                fileWriter.write(DataBlob);
		                resolve("success");
		            }, function(){
		                $log.debug('Unable to save file in path '+ folderpath);
		                reject("error");
		            });
				});
		    });
  		});
	}

	return {
		initImageCache: function() { // Initialisiert Cache und löscht alte Fotos
			var img_cache = new CordovaFileCache({
		      fs: new CordovaPromiseFS({
		        Promise: Promise
		      }),
		      mode: 'hash',
		      localRoot: 'data',
		      cacheBuster: true  // optional
		    });

		    var imgCacheSrv = this;

		    img_cache.ready.then(function(list){
		    	$rootScope.image_cache = img_cache;

		    	imgCacheSrv.checkCacheForExpiredItems();
		    })
		},
		getCacheTimestamps: function() {
			var timestamps = timestampCache.get(CACHE_ROUTE().IMAGE_TIMESTAMPS);

			if(timestamps == null || timestamps == undefined || timestamps.length == null || timestamps.length == undefined){
				timestamps = new Array();
			}

			return timestamps;
		},
		saveCacheTimestamps: function(obj) {
			timestampCache.put(CACHE_ROUTE().IMAGE_TIMESTAMPS, obj);
		},
		// Prüft ob das Bild bereits im Cache ist, wenn ja, wird die URL auf dem Filesystem zurückgegeben
		// Wenn nicht, wird es als "zu cachen" markiert
		cacheImage: function(url) {
			var image_cache = $rootScope.image_cache;
			
			if(image_cache == null || image_cache == undefined){
				$log.debug("image_cache == null || image_cache == undefined");
				return;
			}

			image_cache.add(url);

			return image_cache.get(url);
		},
		// Speichert alle, mittels cacheImage hinzugefügten Bilder
		cacheAllImages: function() {
			if($rootScope.image_cache != null && $rootScope.image_cache != undefined) {
				var onprogress = function(e) {
					var progress ="Progress: "
					+ e.queueIndex // current download index
					+ " "
					+ e.queueSize; // total files to download

					//$log.debug(progress);
				}

				var allTimestamps = this.getCacheTimestamps();
				var timestampObj = {};
				
				timestampObj.timestamp = moment();
				timestampObj.images = $rootScope.image_cache.getDownloadQueue();

				if(timestampObj.images.length > 0)
					allTimestamps.push(timestampObj);

				this.saveCacheTimestamps(allTimestamps);

				$rootScope.image_cache.download(onprogress, true).then(function(cache){
					$log.debug("Caching of images: Success.");
				},function(failedDownloads) {
					$log.debug("Failed to cache images.");
					$log.debug(failedDownloads);
				})	
			}
		},
		checkCacheForExpiredItems: function() {
			if($_network.isOffline) return;

			var timestamps = this.getCacheTimestamps();
			for(var i=timestamps.length-1; i>=0; i--) {
				if(moment(new Date()).diff(moment(timestamps[i].timestamp), 'seconds') >= maxCacheTimeSec) {
					var images = timestamps[i].images;

					for(var x=0; x<=images.length-1; x++) {
						this.removeImage(images[x]);
					}

					timestamps.splice(i,1);
				}
			}

			this.saveCacheTimestamps(timestamps);
		},
		removeImage: function(url) {
			$rootScope.image_cache.remove(url, true).then(function(success){
				$log.debug("Removing image success.");
				$log.debug(success);
			}, function(error){
				$log.debug("Error removing image.");
				$log.debug(error);
			});
		},
		clearImageCache: function() {
			this.saveCacheTimestamps(new Array());

			if($rootScope.image_cache != null && $rootScope.image_cache != undefined) {
				$rootScope.image_cache.clear().then(function(success){
					$log.debug("Image cache cleared");
				});
			}
			
		},
		getAllCachedImages: function() {
			return new Promise(function(resolve, reject){
				$rootScope.image_cache.list().then(function(list){
					resolve(list);
				},function(err){
					$log.debug(err);

					reject(err);
				});
			});
		},
		cacheBase64Image: function(base64String, filename) { // Speichert ein base64 Bild. return: Lokale URL zu dem Bild.
			return new Promise(function(resolve, reject){
				var block = base64String.split(";");
				var dataType = block[0].split(":")[1];
				var realData = block[1].split(",")[1];
				var folderpath = cordova.file.cacheDirectory;

				$log.debug("folderpath");
				$log.debug(folderpath);

				if(folderpath != null) {
					var save = savebase64AsImageFile(folderpath, filename, realData, dataType);

					save.then(function(success){
						resolve(folderpath + '' + filename);
					}, function(error){
						$log.debug(error);
						resolve(error);
					});
				} else {
					resolve(folderpath);
				}

			//return folderpath + '' + filename;
			});
		}
	}
});