angular.module('chat_detail.service', [])
.factory('$_chat_detail', function($_sdk, $_helper, $_chat_detail_cache, $_notification, $filter, $_chat_list_cache, $_image_cache, $sce, $log) {
    var pageURL = "#";
    var onFirstPage = true;
    var onLastPage = false;
    var matchId;
    var isPush = false;

    var getPromise = function(resolve, reject) {
        var callbacks = {
            onSuccessAck: function(successData) {
                createChatMessagesList(successData.data).then(
                    function(chat) {

                        //$_chat_detail_cache.saveChat(matchId, chat);
                        var lastMessage = chat.messages[chat.messages.length-1];

                        if(onFirstPage) {
                            $_chat_list_cache.updateLastMessage(matchId, lastMessage.usr_id, lastMessage.messageWithOutURL);
                        }

                        if(!isPush) getNextPageURL(successData);

                        resolve(chat);
                    },
                    function(error) {
                        reject(error);
                    }
                );
            },
            onFailAck: function(failData) {
                var msg = $filter('translate')('FAILED_TO_LOAD_MESSAGES');
                var type = $_notification.getNotificationTypes().ERROR;

                if(failData.code == 148 || failData.code == 90 || failData.code == 237) {
                    msg = $filter('translate')('CHAT_DOESNT_EXIST_ANYMORE');
                }

                var notificationObj = $_helper.getObjForNotification(msg, type);

                notificationObj.code = failData.code;

                reject(notificationObj);
            },
            onValidator: function(validateData) {
                var msg = $filter('translate')('FAILED_TO_LOAD_MESSAGES');
                var type = $_notification.getNotificationTypes().ERROR;

                reject($_helper.getObjForNotification(msg, type));
            }
        }

        if(isPush) {
            callbacks.onTimeout = function() {
                $log.debug("onTimeoutChat");
                reject(null);
            }
        }

        var params = {
            orders: "sent_time:desc",
            single_entry: 0
        }

        if(onLastPage && !isPush) {
            var msg = $filter('translate')('NO_MORE_DATA_TO_LOAD');
            var type = $_notification.getNotificationTypes().ERROR;

            reject($_helper.getObjForNotification(msg, type));
        } else {
            $log.debug("pageURL: "+pageURL);
            if(!onFirstPage && !isPush) {
                $_sdk.request("GET", pageURL, params, callbacks);
            } else {
                $_sdk.request("GET", "/match/" + matchId + "/messages", params, callbacks);
            }
        }
    }


    function sortChatMessages(messages) {
        return new Promise(function(resolve, reject) {
            messages.sort(function(a,b){
                return a.id - b.id;
            });
            resolve(messages);
        });
    }

    function removeDuplicates(originalArray, prop) {
        var newArray = [];
        var lookupObject  = {};

        for(var i in originalArray) {
            lookupObject[originalArray[i][prop]] = originalArray[i];
        }

        for(i in lookupObject) {
            if(lookupObject[i].message != null && lookupObject[i].message != undefined) {
              newArray.push(lookupObject[i]);  
            }           
        }
        return newArray;
    }

    function createChatMessagesList(data) { // List der erhaltenenen Nachrichten
        return new Promise(function(resolve, reject) {
            var messages = [];
            var messagesData = data.messages;
            var noMessages = messagesData.length;

            var profile_img = $_helper.getSmallImg(data.other_user.profile_images);
            var is_verified = $_helper.isFalseOrTrue(data.other_user.media_verification);

            var infObj = {
                profile_img: profile_img,
                name: data.other_user.name,
                age: data.other_user.age,
                user_id: data.other_user.user_id,
                distance: data.other_user.distance,
                is_verified: is_verified
            }

            for(var i=0; i<=noMessages-1; i++) {
                var msg = messagesData[i];
                var sent_time = msg.sent_time;

                var fullDate = $_helper.newDate(sent_time);
                var short_date = moment(fullDate).format('DD.MM.YY');
                var long_date = $_helper.getUTCOffset(sent_time, 'HH:mm', 2);

                var message_type = "user_message";

                if(msg.system_message) {
                    message_type = "system_message";
                }

                var msgWithURL = $_helper.replaceURLWithHTMLLinks(msg.message_text);
                var messageObj = {
                    id: msg.message_id,
                    usr_id: msg.user_id,
                    message: msgWithURL,
                    messageWithOutURL: msg.message_text,
                    date: long_date,
                    short_date: short_date,
                    sorting_date: fullDate,
                    system_message: msg.system_message,
                    type: message_type,
                    read_time: msg.read_time,
                    temp_id: msg.message_temp_id
                }

                messages.push(messageObj);
            }

            sortChatMessages(messages).then(
                function(sorted) {
                    var chat = {
                        info: infObj,
                        messages: sorted
                    }
                    $_image_cache.cacheAllImages();
                    resolve(chat);
                },
                function(error) {
                    reject(error);
                }
            );
        })
    }

    function getNextPageURL(successData) {
        $_sdk.hasNextPageURL(successData, function(nextPageURL) {
            pageURL = nextPageURL;
            onFirstPage = false;
            onLastPage = false;
        }, function() {
            pageURL = "#";
            onFirstPage = false;
            onLastPage = true;
        });
    }

    return {
        getChat: function(chatId, isPushVal) {
            isPush = (typeof isPushVal === 'undefined') ? false : isPushVal;
            matchId = chatId;
            if(!isPush) {
                pageURL = "#";
                onFirstPage = true;
                onLastPage = false;
            }
            return new Promise(getPromise);
        },
        loadMore: function(chatId) {
            isPush = false;
            matchId = chatId;
            return new Promise(getPromise);
        },
        isOnLastPage: function() {
            return onLastPage;
        },
        prepareMessageArray: function(messages) {
            messages = removeDuplicates(messages, "id");
            messages.sort(function(a,b){
                return a.id - b.id;
            });
            return messages;
        },
        removeSentMessagesFromCache: function(messageArr, chatId) {
            $log.debug("removeSentMessages");
            $log.debug(messageArr);

            var removeIDs = [];
            var messageArrLength = messageArr.length;
            for(var i=0; i<=messageArrLength-1; i++) {
              $log.debug(messageArr[i]);
              if(messageArr[i].temp_id != null) {
                removeIDs.push(messageArr[i].temp_id);
              }
            }

            var removeIDsLength = removeIDs.length;
            for(var i=0; i<=removeIDsLength-1; i++) {
              $log.debug(removeIDs[i]);
              $_chat_detail_cache.removeUnsendMessage(chatId, removeIDs[i]);
            }

        },
        sendMessage: function(id, msg, msgId) {
            return new Promise(function(resolve, reject){
                var callbacks = {
                    onSuccessAck: function(successData) {
                        $_chat_list_cache.updateLastMessage(id, $_sdk.getUserID(), msg);
                        resolve();
                    },
                    onFailAck: function(failData) {
                        $log.debug(failData);
                        var msg = $filter('translate')('FAILED_TO_SEND_MESSAGE');
                        var type = $_notification.getNotificationTypes().ERROR;

                        if(failData.code == 148 || failData.code == 90 || failData.code == 237) {
                            msg = $filter('translate')('CHAT_DOESNT_EXIST_ANYMORE');
                        }

                        var notificationObj = $_helper.getObjForNotification(msg, type);

                        notificationObj.code = failData.code;

                        reject(notificationObj);
                    },
                    onValidator: function(validateData) {
                        var msg = $filter('translate')('FAILED_TO_SEND_MESSAGE');
                        var type = $_notification.getNotificationTypes().ERROR;

                        reject($_helper.getObjForNotification(msg, type));
                    },
                    onTimeout: function(validateData) {
                        var msg = $filter('translate')('FAILED_TO_SEND_MESSAGE');
                        var type = $_notification.getNotificationTypes().ERROR;

                        reject($_helper.getObjForNotification(msg, type));
                    }
                }

                var params = {
                    text: msg,
                    temp_id: msgId
                }

                $_sdk.request("POST", "/match/" + id + "/message", params, callbacks);
            });
        }
    }
});
