angular.module('chat_list.service', [])
.factory('$_chat_list', function($_sdk, $_helper, $_chat_list_cache, $_notification, $filter, $_chat_detail_cache, $_image_cache, $_account_cache, $_location) {
    var pageURL = "#";
    var onFirstPage = true;
    var onLastPage = false;

    /* Führt den Aufruf an die API aus. */
    var getPromise = function(resolve, reject){
        var callbacks = {
            onSuccessAck: function(successData) {
                $_account_cache.setUserHasProfileImage(true);
                createChatList(successData).then(
                    function(chats) {
                        $_chat_list_cache.saveChatList(chats);
                        getNextPageURL(successData);
                        resolve(chats);
                    },
                    function(error) {
                        reject(error);
                    }
                );
            },
            onFailAck: function(failData) {
                $_account_cache.setUserHasProfileImage(true);
                if(failData.code == 46) {
                    $_chat_list_cache.saveChatList({});
                    var msg = $filter('translate')('NO_CHATS_YOU_SHOULD_START_TO_FLIRT');
                    var type = $_notification.getNotificationTypes().ERROR;
                    var notification = $_helper.getObjForNotification(msg, type);

                    notification.code = 46;
                    reject(notification);
                }

                if(failData.code == 265) {
                    $_account_cache.setUserHasProfileImage(false); // nur hier auf false setzen. Rest ist true

                    reject(failData);
                }

                if(failData.code == 271) {
                    reject(failData);
                }

                var msg = $filter('translate')('REQUEST_FAILED');
                var type = $_notification.getNotificationTypes().ERROR;

                reject($_helper.getObjForNotification(msg, type));
            },
            onValidator: function(validateData) {
                $_account_cache.setUserHasProfileImage(true);
                var msg = $filter('translate')('FAILED_TO_LOAD_DATA');
                var type = $_notification.getNotificationTypes().ERROR;

                reject($_helper.getObjForNotification(msg, type));
            }
        }

        var params = {
            includes: "messages_of_match"
        }

        // Hole erst die aktuelle Location des Benutzers
        $_location.getCurrentLocation().then(
            function(locationObj) {
                params.lat = locationObj.lat;
                params.lng = locationObj.lng;

                if(onLastPage) {
                    var msg = $filter('translate')('NO_MORE_DATA_TO_LOAD');
                    var type = $_notification.getNotificationTypes().ERROR;

                    reject($_helper.getObjForNotification(msg, type));
                } else {
                    if(!onFirstPage) {
                        $_sdk.request("GET", pageURL, params, callbacks);
                    } else {
                        $_sdk.request("GET", "/matches", params, callbacks);
                    }
                }
            },
            function(error) {
                if(onLastPage) {
                    var msg = $filter('translate')('NO_MORE_DATA_TO_LOAD');
                    var type = $_notification.getNotificationTypes().ERROR;

                    reject($_helper.getObjForNotification(msg, type));
                } else {
                    if(!onFirstPage) {
                        $_sdk.request("GET", pageURL, params, callbacks);
                    } else {
                        $_sdk.request("GET", "/matches", params, callbacks);
                    }
                }
            }
        )
    }

    /* Erstellt die Chatliste */
    function createChatList(data) {
        return new Promise(function(resolve, reject) {
            var chats = [];
            var chatListPusher = function(item, index) {
                try {
                    var profile_img = $_helper.getSmallImg(item.profile_images);
                    var is_verified = $_helper.isFalseOrTrue(item.media_verification);

                    if(item.message_sent_time) {
                        var clear_date = $_helper.newDate(item.message_sent_time);
                        var message_time = moment(clear_date).format("DD.MM.YY");
                    } else {
                        var messsage_time = "";
                    }

                    var chatObj = {
                        user_id: item.user_id,
                        age: item.age,
                        match_id: item.match_id,
                        name: item.name,
                        profile_img: profile_img,
                        unread_messages: item.my_unread_messages_count,
                        last_message_text: item.message_text,
                        last_message_user_id: item.message_sender_user_id,
                        last_message_sent: message_time,
                        last_message_sent_order: item.message_sent_time,
                        is_verified: is_verified
                    }

                    if(chatObj.unread_messages > 0) {
                        $_chat_detail_cache.markChatAsUnRead(chatObj.match_id, chatObj.unread_messages);
                    }

                    chats.push(chatObj);

                    if(data.from + index == data.to || index == data.total - 1 || index == data.per_page-1) {
                        $_image_cache.cacheAllImages();
                        resolve(chats);
                    } else if(index > data.total) {
                        var msg = $filter('translate')('OOPS_SOMETHING_WENT_WRONG');
                        var type = $_notification.getNotificationTypes().ERROR;

                        reject($_helper.getObjForNotification(msg, type));
                    }
                } catch(e) {
                    // statements
                    var msg = $filter('translate')('OOPS_SOMETHING_WENT_WRONG');
                    var type = $_notification.getNotificationTypes().ERROR;
                    var notifcationObj = $_helper.getObjForNotification(msg, type);
                    notifcationObj.code = 0;

                    reject(notifcationObj);
                    return;
                }

            };

            $_sdk.iterator(data, chatListPusher);
        });
    }

    /* Gibt die nächste Seite des Aufrufs zurück (sofern vorhanden) */
    function getNextPageURL(successData) {
        $_sdk.hasNextPageURL(successData, function(nextPageURL) {
            pageURL = nextPageURL;
            onFirstPage = false;
            onLastPage = false;
        }, function() {
            pageURL = "#";
            onFirstPage = false;
            onLastPage = true;
        });
    }

    return {
        getChats: function() {
            pageURL = "#";
            onFirstPage = true;
            onLastPage = false;
            return new Promise(getPromise);
        },
        loadMore: function() {
            return new Promise(getPromise);
        },
        isOnLastPage: function() {
            return onLastPage;
        },
        isOnFirstPage: function() {
            return onFirstPage;
        },
        setToLastPage: function() {
            onLastPage = true;
        }
    }
});
