angular.module('flirt_list.service', [])
.factory('$_flirt_list', function($_sdk, $_helper, $_flirt_list_cache, $_notification, $filter, $_image_cache, $_account_cache, $log, $_location) {
    var pageURL = "#";
    var onFirstPage = true;
    var onLastPage = false;

    var getPromise = function(resolve, reject){
        var callbacks = {
            onSuccessAck: function(successData) {
                $_account_cache.setUserHasProfileImage(true);
                $log.debug(successData);
                createFlirtList(successData).then(
                    function(flirts) {
                        $_flirt_list_cache.saveFlirtList(flirts);
                        getNextPageURL(successData);
                        resolve(flirts);
                    },
                    function(error) {
                        reject(error);
                    }
                );
            },
            onFailAck: function(failData) {
                $_account_cache.setUserHasProfileImage(true);
                if(failData.code == 46) {
                    $_flirt_list_cache.saveFlirtList({});
                    var msg = $filter('translate')('NO_RESULTS_YOU_SHOULD_START_A_FLIRT');
                    var type = $_notification.getNotificationTypes().ERROR;
                    var notification = $_helper.getObjForNotification(msg, type);

                    notification.code = 46;
                    reject(notification);
                }

                if(failData.code == 265) {
                    $_account_cache.setUserHasProfileImage(false); // nur hier auf false setzen. Rest ist true

                    reject(failData);
                }

                if(failData.code == 271) {
                    reject(failData);
                }

                var msg = $filter('translate')('REQUEST_FAILED');
                var type = $_notification.getNotificationTypes().ERROR;

                reject($_helper.getObjForNotification(msg, type));
            },
            onValidator: function(validateData) {
                $_account_cache.setUserHasProfileImage(true);
                var msg = $filter('translate')('FAILED_TO_LOAD_DATA');
                var type = $_notification.getNotificationTypes().ERROR;

                reject($_helper.getObjForNotification(msg, type));
            }
        }

        // Hole erst die aktuelle Location des Benutzers
        $_location.getCurrentLocation().then(
            function(locationObj) {
                var params = {
                    lat: locationObj.lat,
                    lng: locationObj.lng
                }

                if(onLastPage) {
                    var msg = $filter('translate')('NO_MORE_DATA_TO_LOAD');
                    var type = $_notification.getNotificationTypes().ERROR;

                    reject($_helper.getObjForNotification(msg, type));
                } else {
                    if(!onFirstPage) {
                        $_sdk.request("GET", pageURL, params, callbacks);
                    } else {
                        $_sdk.request("GET", "/flirts", params, callbacks);
                    }
                }
            },
            function(error) {
                if(onLastPage) {
                    var msg = $filter('translate')('NO_MORE_DATA_TO_LOAD');
                    var type = $_notification.getNotificationTypes().ERROR;

                    reject($_helper.getObjForNotification(msg, type));
                } else {
                    if(!onFirstPage) {
                        $_sdk.request("GET", pageURL, null, callbacks);
                    } else {
                        $_sdk.request("GET", "/flirts", null, callbacks);
                    }
                }
            }
        )
    }

    function createFlirtList(data) {
        return new Promise(function(resolve, reject) {
            var flirts = [];
            var flirtListPusher = function(item, index) {
                //item.flirt_round_expired = true;
                try {
                    if(item.my_round_status  == 'complete') { // Überspringt die Spiele, die "completed" sind, weil sie bereits als Matches angezeigt werden
                        if(index == data.per_page - 1 || data.from + index == data.to || index == data.to - 1) {
                            resolve(sortFlirtList(flirts));
                        };
                    } else {
                        var profile_img = $_helper.getSmallImg(item.profile_images);
                        var is_verified = $_helper.isFalseOrTrue(item.media_verification);

                        var flirtObj = {
                            flirt_id: item.flirt_id,
                            profile_img: profile_img,
                            name: item.name,
                            age: item.age,
                            hashtag1: item.hashtag1,
                            hashtag2: item.hashtag2,
                            hashtag3: item.hashtag3,
                            my_round_status: item.my_round_status,
                            round: item.round,
                            user_id: item.other_user_id,
                            flirt_round_valid_until: item.flirt_round_valid_until,
                            flirt_round_expired: item.flirt_round_expired,
                            //is_active: item.is_active,
                            is_verified: is_verified,
                            is_deleted_by_other_user: item.is_deleted_by_other_user
                        }

                        flirts.push(flirtObj);

                        if(data.from + index == data.to || index == data.to - 1 || index == data.per_page-1) {
                            $_image_cache.cacheAllImages();
                            resolve(sortFlirtList(flirts));
                        } else if(index > data.total) {
                            var msg = $filter('translate')('OOPS_SOMETHING_WENT_WRONG');
                            var type = $_notification.getNotificationTypes().ERROR;

                            reject($_helper.getObjForNotification(msg, type));
                        }
                    }
                } catch(e) {
                    var msg = $filter('translate')('OOPS_SOMETHING_WENT_WRONG');
                    var type = $_notification.getNotificationTypes().ERROR;
                    var notificationObj = $_helper.getObjForNotification(msg, type);

                    notificationObj.code = 0;
                    reject(notificationObj);
                }


            };

            $_helper.moveArrayElement();
            $_sdk.iterator(data, flirtListPusher);
        });
    }

    // Sortiert die Flirtliste, sodass am Ende die verlorenen Spiele angezeigt werden
    function sortFlirtList(flirts) {
        for(var j=0; j<=flirts.length-1; j++) {
            for(var i=0; i<=flirts.length-1; i++) {
                if(flirts[i].my_round_status == 'gameover' || flirts[i].my_round_status == 'deleted' || flirts[i].flirt_round_expired == true) {
                    flirts.move(i, flirts.length-1);
                }
            }
        }
        return flirts;
    }

    function getNextPageURL(successData) {
        $_sdk.hasNextPageURL(successData, function(nextPageURL) {
            pageURL = nextPageURL;
            onFirstPage = false;
            onLastPage = false;
        }, function() {
            pageURL = "#";
            onFirstPage = false;
            onLastPage = true;
        });
    }

    return {
        getFlirts: function() {
            pageURL = "#";
            onFirstPage = true;
            onLastPage = false;
            return new Promise(getPromise);
        },
        loadMore: function() {
            return new Promise(getPromise);
        },
        isOnLastPage: function() {
            return onLastPage;
        },
        isOnFirstPage: function() {
            return onFirstPage;
        },
        sortFlirtList(flirts) {
            return sortFlirtList(flirts);
        }
    }
});
