// Diese Service Klasse dient für die Umsetzung der Calls für die Optionen bei den Spielen und Chats.
angular.module('options.service', [])
.factory('$_options', function($_sdk, $filter, $_notification, $_helper, $_users_cache, $_blocked_users_cache, $log) {
    function createReportReasonsList(data){
        return new Promise(function(resolve, reject){
            var reasons = [];

            var reportReasonsPusher = function(item, index) {
                var reasonObj = {
                    id: item.report_types_id,
                    name: item.report_types_name_translated,
                    report_types_translation_key_for_name: item.report_types_translation_key_for_name
                }

                reasons.push(reasonObj);

                if(index == data.total - 1) {
                    resolve(reasons);
                } else if(index > data.total) {
                    var msg = $filter('translate')('OOPS_SOMETHING_WENT_WRONG');
                    var type = $_notification.getNotificationTypes().ERROR;

                    reject($_helper.getObjForNotification(msg, type));
                }
            };

            $_sdk.iterator(data, reportReasonsPusher);
        });
    }

    function createDeleteReasonsList(data){
        return new Promise(function(resolve, reject){
            var reasons = [];

            var reportReasonsPusher = function(item, index) {
                var reasonObj = {
                    id: item.user_delete_types_id,
                    name: item.user_delete_types_name_translated
                }

                reasons.push(reasonObj);

                if(index == data.total - 1) {
                    resolve(reasons);
                } else if(index > data.total) {
                    var msg = $filter('translate')('OOPS_SOMETHING_WENT_WRONG');
                    var type = $_notification.getNotificationTypes().ERROR;

                    reject($_helper.getObjForNotification(msg, type));
                }
            };

            $_sdk.iterator(data, reportReasonsPusher);
        });
    }

    function deleteMatch(flirtId) {
      return new Promise(function(resolve, reject) {
        var callbacks = {
          onSuccessAck: function(successData) {
            var msg = $filter('translate')('CHAT_SUCCESSFULLY_DELETED');
            var type = $_notification.getNotificationTypes().POSITIVE;

            resolve($_helper.getObjForNotification(msg, type));
          },
          onFailAck: function(failData) {
            var msg = $filter('translate')('FAILED_TO_DELETE_CHAT');
            var type = $_notification.getNotificationTypes().ERROR;

            reject($_helper.getObjForNotification(msg, type));
          },
          onValidator: function(validateData) {
            var msg = $filter('translate')('FAILED_TO_DELETE_CHAT');
            var type = $_notification.getNotificationTypes().ERROR;

            reject($_helper.getObjForNotification(msg, type));
          }
        }

        $_sdk.request("DELETE", "/match/" + flirtId, null, callbacks);
      });
    }

    function deleteFlirt(flirtId) {
      return new Promise(function(resolve, reject){
          var callbacks = {
              onSuccessAck: function(successData) {
                  var msg = $filter('translate')('GAME_SUCCESSFULLY_DELETED');
                  var type = $_notification.getNotificationTypes().POSITIVE;

                  resolve($_helper.getObjForNotification(msg, type));
              },
              onFailAck: function(failData) {
                  var msg = $filter('translate')('FAILED_TO_DELETE_GAME');
                  var type = $_notification.getNotificationTypes().ERROR;

                  reject($_helper.getObjForNotification(msg, type));
              },
              onValidator: function(validateData) {
                  var msg = $filter('translate')('FAILED_TO_DELETE_GAME');
                  var type = $_notification.getNotificationTypes().ERROR;

                  reject($_helper.getObjForNotification(msg, type));
              }
          }

          $_sdk.request("DELETE", "/flirt/" + flirtId, null, callbacks);
      });
    }

    function deleteFailedFlirts() {
      return new Promise(function(resolve, reject){
          var callbacks = {
              onSuccessAck: function(successData) {
                  var msg = $filter('translate')('FAILED_FLIRTS_SUCCESSFULLY_DELETED');
                  var type = $_notification.getNotificationTypes().POSITIVE;

                  resolve($_helper.getObjForNotification(msg, type));
              },
              onFailAck: function(failData) {
                  var msg = $filter('translate')('FAILED_TO_DELETE_GAME');
                  var type = $_notification.getNotificationTypes().ERROR;

                  reject($_helper.getObjForNotification(msg, type));
              },
              onValidator: function(validateData) {
                  var msg = $filter('translate')('FAILED_TO_DELETE_GAME');
                  var type = $_notification.getNotificationTypes().ERROR;

                  reject($_helper.getObjForNotification(msg, type));
              }
          }

          $_sdk.request("DELETE", "/flirts/failed", null, callbacks);
      });
    }
    

    function blockUser(userId) {
      return new Promise(function(resolve, reject){
                var callbacks = {
                    onSuccessAck: function(successData) {
                        var userObj = {
                            block_id: successData.block_id,
                            user_id: userId,
                            profile_img: "",
                            name: ""
                        }

                        $_users_cache.deleteUser(userId);

                    	var msg = $filter('translate')('USER_SUCCESSFULLY_BLOCKED');
						var type = $_notification.getNotificationTypes().POSITIVE;
                    	resolve($_helper.getObjForNotification(msg, type));
                    },
                    onFailAck: function(failData) {
                    	$log.debug(failData);
                        var msg = $filter('translate')('FAILED_TO_BLOCK_USER');
                        var type = $_notification.getNotificationTypes().ERROR;
                        if(failData.code == 273) {
                          reject($_helper.getObjForNotification($filter('translate')('YOU_CANNOT_IGNORE_BLURY'), type));
                        } else {
                          reject($_helper.getObjForNotification(msg, type));
                        }
                        reject($_helper.getObjForNotification(msg, type));
                    },
                    onValidator: function(validateData) {
                        var msg = $filter('translate')('FAILED_TO_BLOCK_USER');
                        var type = $_notification.getNotificationTypes().ERROR;

                        reject($_helper.getObjForNotification(msg, type));
                    }
                }

                $_sdk.request("POST", "/user/" + userId + "/block", null, callbacks);
            });
    }

    function unblockUser(user_id) {
      return new Promise(function(resolve, reject){
          var callbacks = {
              onSuccessAck: function(successData) {
                  $_blocked_users_cache.removeBlockedUser(user_id);


                  var msg = $filter('translate')('USER_SUCCESSFULLY_UNBLOCKED');
                  var type = $_notification.getNotificationTypes().POSITIVE;
                  resolve($_helper.getObjForNotification(msg, type));
              },
              onFailAck: function(failData) {
                  var msg = $filter('translate')('FAILED_TO_UNBLOCK_USER');
                  var type = $_notification.getNotificationTypes().ERROR;

                  reject($_helper.getObjForNotification(msg, type));
              },
              onValidator: function(validateDate) {
                  var msg = $filter('translate')('FAILED_TO_UNBLOCK_USER');
                  var type = $_notification.getNotificationTypes().ERROR;

                  reject($_helper.getObjForNotification(msg, type));
              }
          }

          $_sdk.request("DELETE", "/user/" + user_id + "/block", null, callbacks);
      });
    }

    function getReportReasons(translation_key) {
      return new Promise(function(resolve, reject){
          var callbacks = {
              onSuccessAck: function(successData) {
                  createReportReasonsList(successData).then(
                      function(list){
                          resolve(list);
                      },
                      function(error){
                          reject(error);
                      }
                  );
              },
              onFailAck: function(failData) {
                  $log.debug(failData);
                  var msg = $filter('translate')('FAILED_TO_REPORT_USER');
                  var type = $_notification.getNotificationTypes().ERROR;

                  reject($_helper.getObjForNotification(msg, type));
              },
              onValidator: function(validateData) {
                  $log.debug(validateData);
                  var msg = $filter('translate')('FAILED_TO_REPORT_USER');
                  var type = $_notification.getNotificationTypes().ERROR;

                  reject($_helper.getObjForNotification(msg, type));
              }
          }

          $_sdk.request("GET", "/report/types", null, callbacks);
      });
    }

    function reportUser(userId, reasonId) {
      return new Promise(function(resolve, reject){
          var callbacks = {
              onSuccessAck: function(successData) {
                  var msg = $filter('translate')('USER_SUCCESSFULLY_REPORTED');
                  var type = $_notification.getNotificationTypes().POSITIVE;

                  resolve($_helper.getObjForNotification(msg, type));
              },
              onFailAck: function(failData) {
                  var msg = $filter('translate')('FAILED_TO_REPORT_USER');
                  var type = $_notification.getNotificationTypes().ERROR;
                  if(failData.code == 273) {
                    $log.debug("YOU_CANNOT_REPORT_BLURY");
                    reject($_helper.getObjForNotification($filter('translate')('YOU_CANNOT_REPORT_BLURY'), type));
                  } else {
                    $log.debug("!YOU_CANNOT_REPORT_BLURY");
                    reject($_helper.getObjForNotification(msg, type));
                  }
              },
              onValidator: function(validateData) {
                  $log.debug(validateData);
                  var msg = $filter('translate')('FAILED_TO_REPORT_USER');
                  var type = $_notification.getNotificationTypes().ERROR;
                  reject($_helper.getObjForNotification(msg, type));
              }
          }

          var params = {
              report_types_id: reasonId
          }

          $_sdk.request("POST", "/user/" + userId + "/report", params, callbacks);
      });
    }

    function getDeleteReasons() {
      return new Promise(function(resolve, reject){
          var callbacks = {
              onSuccessAck: function(successData) {
                  createDeleteReasonsList(successData).then(function(list){
                      resolve(list);
                  }, function(error){
                      reject(error);
                  });
              },
              onFailAck: function(failData) {
                  $log.debug(failData);
                  var msg = $filter('translate')('FAILED_TO_REPORT_USER');
                  var type = $_notification.getNotificationTypes().ERROR;

                  reject($_helper.getObjForNotification(msg, type));
              },
              onValidator: function(validateData) {
                  $log.debug(validateData);
                  var msg = $filter('translate')('FAILED_TO_REPORT_USER');
                  var type = $_notification.getNotificationTypes().ERROR;

                  reject($_helper.getObjForNotification(msg, type));
              }
          }

          $_sdk.request("GET", "/delete/types", null, callbacks);
      });
    }

    return {
      deleteMatch: function(flirtId) { // Ein Match = Ein Chat, für Spiele siehe deleteFlirt
        return deleteMatch(flirtId);
      },
      deleteFlirt: function(flirtId) {
        return deleteFlirt(flirtId);
      },
      deleteFailedFlirts: function() {
        return deleteFailedFlirts();
      },
      blockUser: function(userId) {
        return blockUser(userId);
      },
      unblockUser: function(user_id) {
        return unblockUser(user_id);
      },
      getReportReasons: function(translation_key) {
        return getReportReasons(translation_key);
      },
      reportUser: function(userId, reasonId) {
        return reportUser(userId, reasonId);
      },
      getDeleteReasons: function() {
        return getDeleteReasons();
      }
  	};
});
