angular.module('upload.service', [])
.factory('$_upload', function($_sdk, $_helper, $filter, $_notification, $timeout, $ionicPopup, $ionicActionSheet, $log) {
	return {

		uploadProfileImage(mediaTypeID, src, scope) {
			return new Promise(function(resolve, reject){
                $_sdk.setObjectID($_sdk.getUserID());
                $_sdk.upload(mediaTypeID, src, function(currentStatus){
                    $_sdk.debug(currentStatus + "%", "upload");
                    $timeout(function(){
                        scope.load.percent = parseInt(currentStatus);
                    }, 25);
                    if(parseInt(currentStatus) == 100) {
                        var msg = $filter('translate')('UPLOAD_SUCCESS');
                        var type = $_notification.getNotificationTypes().POSITIVE;

                        scope.load.percent = 0;
                        resolve($_helper.getObjForNotification(msg, type));
                    }
                }, function(response){
                    var msg = $filter('translate')('UPLOAD_SUCCESS');
                    var type = $_notification.getNotificationTypes().POSITIVE;

                    scope.load.percent = 0;
                    resolve($_helper.getObjForNotification(msg, type));
                }, function(error) {
                    var msg = $filter('translate')('UPLOAD_SUCCESS');
                    var type = $_notification.getNotificationTypes().ERROR;

                    scope.load.percent = 0;
                    reject($_helper.getObjForNotification(error, type));
                });
            });
		},

        uploadPopup(scope) {
            $ionicPopup.show({
                template: 
                    '<ion-spinner icon="ios"></ion-spinner>'
                    + '<br/>'
                    + '<div class="loading-text">'
                    + $filter('translate')('PICTURE_UPLOAD_IN_PROGRESS_TEXT')
                    + '<br/><strong>{{load.percent}}%</strong></div>',
                title: $filter('translate')('PICTURE_UPLOAD_IN_PROGRESS_TITLE'),
                cssClass: 'confirm-popup',
                scope: scope,
            });
        },

        // Onur: Die Methoden die ich nachträglich hinzugefügt habe am 31.08.17

        uploadFile(objectId, mediaTypeID, src, onUploadProgress) {
            return new Promise(function(resolve, reject){
                $_sdk.setObjectID(objectId);
                $_sdk.upload(mediaTypeID, src, function(currentStatus){
                    if(typeof onUploadProgress === "function") onUploadProgress(currentStatus);
                }, function(response){
                    resolve(response);
                }, function(error) {
                    reject(error);
                });
            });
        },

        uploadViaDownloadUrl: function (objectID, mediaTypeID, downloadUrl) {
          return new Promise(function (resolve, reject) {
            var callbacks = {
              onSuccessAck: function (successData) {
                resolve(successData);
              },
              onFailAck: function (failData) {
                reject(failData);
              },
              onValidator: function (validateData) {
                reject(validateData);
              }
            };
            $_sdk.setObjectID(objectID);
            $_sdk.uploadViaDownloarUrl(mediaTypeID, downloadUrl, callbacks);
          });
        },

        getImage: function(imageSourceType) {

          return new Promise(function(resolve, reject) {

            navigator.camera.getPicture(function(imageUri) {
              resolve(imageUri);
            }, function(message) {
              reject(message);
            }, {
              quality: 100,
              destinationType: Camera.DestinationType.FILE_URI,
              sourceType: imageSourceType,
              correctOrientation: true
            });

          });

        },

        resizeImage: function(longSideMax, url, callback) {
          $log.debug("URL1: " + url);
          var tempImg = new Image();
          tempImg.src = url;
          tempImg.onload = function() {
            // Get image size and aspect ratio.
            var targetWidth = tempImg.width;
            var targetHeight = tempImg.height;
            var aspect = tempImg.width / tempImg.height;

            // Calculate shorter side length, keeping aspect ratio on image.
            // If source image size is less than given longSideMax, then it need to be
            // considered instead.
            if (tempImg.width > tempImg.height) {
              longSideMax = Math.min(tempImg.width, longSideMax);
              targetWidth = longSideMax;
              targetHeight = longSideMax / aspect;
            }
            else {
              longSideMax = Math.min(tempImg.height, longSideMax);
              targetHeight = longSideMax;
              targetWidth = longSideMax * aspect;
            }

            // Create canvas of required size.
            var canvas = document.createElement('canvas');
            canvas.width = targetWidth;
            canvas.height = targetHeight;

            var ctx = canvas.getContext("2d");
            // Take image from top left corner to bottom right corner and draw the image
            // on canvas to completely fill into.
            ctx.drawImage(this, 0, 0, tempImg.width, tempImg.height, 0, 0, targetWidth, targetHeight);

            $log.debug("URL2: " + canvas.toDataURL("image/jpeg"));

            callback(canvas.toDataURL("image/jpeg"));
          };
        },

        uploadFileActionSheet: function() {

          return new Promise(function(resolve, reject) {

            var hideSheet = $ionicActionSheet.show({
              buttons: [
                { text: $filter('translate')('TAKE_PHOTO_FROM_CAM') },
                { text: $filter('translate')('TAKE_PHOTO_FROM_BIB') }
              ],
              titleText:  $filter('translate')('ADD_MEDIA'),
              cancelText: $filter('translate')('CANCEL'),
              cancel: function() {},
              buttonClicked: function(index) {
                hideSheet();
                if (index == 0) {
                  resolve("camera");
                } else if(index == 1) {
                  resolve("gallery");
                } else {
                  reject("Unknown Index");
                }
              }
            });

          });

        }

	};
});
