angular.module('language.service', [])
.factory('$_language', function($_sdk, $translate, $_language_cache, LOCALES) {
  /* Gibt die von der API zur Verfügung gestellten Sprachen zurück. */
  function getLanguages() {
    return new Promise(function(resolve, reject){
      var callbacks = {
        onSuccessAck: function(successData) {
          resolve(successData.data);
        },
        onFailAck: function(failData) {
          reject(failData);
        },
        onValidator: function(validateData) {
          reject(validateData);
        }
      }

      $_sdk.request("GET", "/language", null, callbacks);
    });
  }

  /* Gibt das Kürzel der aktuell benutzten Gerätesprache zurück. */
  function getLocaleKey() {
    var use = $translate.use();

    for(var i=0; i <= LOCALES.locales.length - 1; i++) {
      if(LOCALES.locales[i].abbr == use) {
        return LOCALES.locales[i].key;
      }
    }
  }

  /* Setzt die Sprache der App auf die übergebene Sprache */
  function setLanguage(language_id, key, languages_name){
    $_sdk.setLanguageID(language_id);
    $translate.use(key).then(function (key) {});
    moment.locale(key);

    var langObj = {
      id: language_id,
      key: key,
      languages_name: languages_name
    }
    $_language_cache.saveLanguageSettings(langObj);
  }

  /* Setzt die Sprache der Anwendung auf die im Cache gespeicherte Sprache. */
  function setCacheLanguage() {
    var langObj = $_language_cache.getLanguageSettings();
    var currentLang = $translate.proposedLanguage() || $translate.use();

    var locales = LOCALES.locales;

    if(langObj != null && langObj != undefined) { // Setze Language aus Key
      setLanguage(langObj.id, langObj.key, langObj.languages_name);
      return;
    } else { // Setze language entsprechend des Geräts
      for(var i=0; i<=locales.length-1; i++) {
        if(locales[i].abbr == currentLang) {
          setLanguage(locales[i].language_id, locales[i].abbr, locales[i].name);
          return;
        }
      }
    }

    // Fallback, sollte nie hierher kommen
    setLanguage(1, "de", "Deutsch");
  }

  return {

    getLocaleKey: function() {
      return getLocaleKey();
    },
    getLanguages: function() {
      return getLanguages();
    },
    setLanguage: function(language_id, key, languages_name){ // Setzt die Sprache auf die Id und den Key
      setLanguage(language_id, key, languages_name);
    },
    setCacheLanguage: function() {
      setCacheLanguage();
    }
  };
});
