angular.module('location.service', [])
.factory('$_location', function($_sdk, $_helper, $filter, $_notification, $cordovaGeolocation) {

    var currentLat = 0;
    var currentLng = 0;
    /* Berechnet den Abstand zwischen zwei geographischen Punkten */
    function getDistance(my_lat, my_lng, other_lat, other_lng) {
        var distance;

        distance =
        (6371 * Math.acos(Math.cos( radians(my_lat))
               * Math.cos(radians(other_lat))
               * Math.cos(radians(other_lng) - radians(my_lng))
               + Math.sin(radians(my_lat))
               * Math.sin(radians(other_lat))));

        return distance;
    }

    function radians(degrees) {
        var TAU = 2 * Math.PI;
        return degrees * TAU / 360;
    }

    
    

    /* Prüft ob die Anwendung die Berechtigung hat die aktuelle GPS-Position auszulesen. */
    function hasLocationPermissions(){
        cordova.plugins.diagnostic.isLocationAuthorized(function(authorized){
            return authorized;
        }, function(error){
            return false;
        })
    }

    return {

      /* Schickt die aktuelle Position an den Server */
      setCurrentLocation() {
        return new Promise(function(resolve, reject){
          this.getCurrentLocation().then(
              function(locationObj){
                  var callbacks = {
                      onSuccessAck: function(successData) {
                          resolve(successData);
                      },
                      onFailAck: function(failData) {
                          reject(failData);
                      },
                      onValidator: function(validateData) {
                          reject(validateData);
                      }
                  }

                  var params = {
                      lat: locationObj.lat,
                      lng: locationObj.lng
                  }

                  $_sdk.request("POST", "/user/location", params, callbacks);
              },
              function(error){
                  reject(error)
              }
            );
        });
      },
      /* Gibt die aktuellen GPS-Koordinaten zurück */
      getCurrentLocation() {
        return new Promise(function(resolve, reject){
          var onSuccess = function(position) {
            currentLat = position.coords.latitude;
            currentLng = position.coords.longitude;
            var coordObj = {
                lat: position.coords.latitude,
                lng: position.coords.longitude
            }
            resolve(coordObj);
          };

          var onError = function(error) {

            cordova.plugins.diagnostic.isLocationAuthorized(function(authorized){
                if(authorized) {
                    // Fallback-Location
                    if(currentLat != undefined && currentLat != null && currentLat != 0 && currentLng != undefined && currentLng != null && currentLng != 0) {

                        var coordObj = {
                            lat: currentLat,
                            lng: currentLng
                        }
                        resolve(coordObj);

                    } else {
                        var msg = $filter('translate')('COULD_NOT_READ_GPS_COORDINATES');
                        var type = $_notification.getNotificationTypes().ERROR;
                        var notificationObj = $_helper.getObjForNotification(msg, type);
                        notificationObj.errorType = "COULD_NOT_READ_GPS_COORDINATES";
                        notificationObj.code = -1;
                        reject(notificationObj);
                    }

                } else {
                    var msg = $filter('translate')('GRANT_GPS_PERMISSIONS');
                    var type = $_notification.getNotificationTypes().ERROR;
                    var notificationObj = $_helper.getObjForNotification(msg, type);
                    notificationObj.errorType = "GRANT_GPS_PERMISSIONS";

                    if(cordova.platformId === 'android') {
                        cordova.plugins.diagnostic.requestLocationAuthorization(function(status){

                        }, function(error){

                        });
                    }

                    notificationObj.code = -1;
                    reject(notificationObj);
                }
            }, function(error){
                var msg = $filter('translate')('GRANT_GPS_PERMISSIONS');
                var type = $_notification.getNotificationTypes().ERROR;
                var notificationObj = $_helper.getObjForNotification(msg, type);
                notificationObj.errorType = "GRANT_GPS_PERMISSIONS";

                if(cordova.platformId === 'android') {
                    cordova.plugins.diagnostic.requestLocationAuthorization(function(status){

                    }, function(error){

                    });
                }

                notificationObj.code = -1;
                reject(notificationObj);
            });

            
             
            
          };

          cordova.plugins.diagnostic.isLocationEnabled(
              function(enabled){
                  //navigator.geolocation.getCurrentPosition(onSuccess, onError, {timeout: 10000});
                  var posOptions = {timeout: 5000, enableHighAccuracy: false};
                  /*$cordovaGeolocation
                      .getCurrentPosition(posOptions)
                      .then(onSuccess, onError);*/

                  navigator.geolocation.getCurrentPosition(onSuccess, onError, posOptions);
              },
              function(error){
                  var msg = $filter('translate')('GPS_NOT_ENABLED');
                  var type = $_notification.getNotificationTypes().ERROR;
                  var notificationObj = $_helper.getObjForNotification(msg, type);
                  notificationObj.errorType = "GPS_NOT_ENABLED";

                  notificationObj.code = -1;
                  reject(notificationObj);
              });

        });
      },
      getDistance(my_lat, my_lng, other_lat, other_lng) {
          return getDistance(my_lat, my_lng, other_lat, other_lng);
      },
      hasLocationPermissions() {
        return hasLocationPermissions();
      },
      requestLocationAuthorization() {
        return new Promise(function(resolve, reject){
          cordova.plugins.diagnostic.requestLocationAuthorization(function(status){
            switch(status){
              case cordova.plugins.diagnostic.permissionStatus.NOT_REQUESTED:
                if(hasLocationPermissions()) resolve();
                else reject(true);
              break;
              case cordova.plugins.diagnostic.permissionStatus.GRANTED:
                resolve();
              break;
              case cordova.plugins.diagnostic.permissionStatus.DENIED:
                if(cordova.platformId === 'android') reject();
                if(cordova.platformId === 'ios') reject(true);
              break;
              case cordova.plugins.diagnostic.permissionStatus.DENIED_ALWAYS:
                reject(true);
              break;
              case cordova.plugins.diagnostic.permissionStatus.GRANTED_WHEN_IN_USE:
                if(hasLocationPermissions()) resolve();
                else reject(true);
              break;
            }
          }, function(error){
              console.error(error);
          });
        });
      }
    };
});
