angular.module('social_account.service', [])
.factory('$_social_account', function($_sdk, $_flirt_detail_cache, $_helper, $filter, $_notification, $cordovaOauth, $http, $_account_cache, $log, $_nativestorage, $ionicPopup) {
	var access_token;

		/*
		 * Facebook login, überspringt bei type=login die Registrierung und loggt User ein (falls Account vorhanden)
		 * @param {string} type - login|register Typ der Registrierung
		 */
    function processFacebookRegistration(type) {
    	return new Promise(function(resolve, reject){
            getFacebookData().then(function(result){
                if(type == 'login') {
                    var user = result.data;
                    var login = loginFacebook(user.id, access_token);
                    
                    login.then(
                        function(success){
                            resolve(success);
                        },
                        function(error){
                            reject(error);
                        }
                        );
                    
                } else {
                    var user = result.data;

                    $log.debug("Register facebook");

                    registerFacebook(user.id, access_token, user.email).then(
                        function (success) {
                            $log.debug(success);
                            resolve(success);
                        },
                        function (error) {
                            reject(error);
                        }
                        );
                };
            }, function(error){
                $log.debug(error);
                reject(error);
            })
    	});
    };

		/* Holt den Token und Daten bei Facebook. */
    function getFacebookData() {
        return new Promise(function(resolve, reject){
            $cordovaOauth.facebook("240022349831838", ["email", "public_profile"], {
                redirect_uri: "https://localhost/callback"
            }).then(function (result) {
                access_token = result.access_token;

                $http.get("https://graph.facebook.com/v3.0/me", {
                    params: {
                        access_token: result.access_token,
                        fields: "id,email",
                        format: "json"
                    }
                }).then(function (result) {
                    var user = result.data;

                    $log.debug("result data");
                    $log.debug(result);

                    resolve(result);
                });

            }, function (error) {
                $log.debug(error);
            });
        });
    }

    function registerFacebook(platform_id, platform_token, platform_email) {
    	return new Promise(function(resolve, reject){
    		var callbacks = {
    			onSuccessAck: function(successData) {
    				$log.debug(successData);
    				var msg = $filter('translate')('SUCCESSFULLY_REGISTERED');
    				var type = $_notification.getNotificationTypes().POSITIVE;

    				var notificationObj = $_helper.getObjForNotification(msg, type);

    				resolve(notificationObj);
    			},
    			onFailAck: function(failData) {
    				$log.debug(failData);
                    if(failData.code == 275) {
                        var msg = $filter('translate')('FACEBOOK_ACCOUNT_ALREADY_IN_USE');
                    } else {
                        var msg = $filter('translate')('FAILED_TO_REGISTER');
                    }
    				
    				var type = $_notification.getNotificationTypes().ERROR;

    				var notificationObj = $_helper.getObjForNotification(msg, type);

    				notificationObj.code = failData.code;
    				reject(notificationObj);
    			},
    			onValidator: function(validateData) {
    				$log.debug(validateData);
                    var msg;
                    var type;

                    if(validateData.platform_email != null && validateData.platform_email != undefined && validateData.platform_email.length > 0) {
                        $log.debug(validateData.platform_email);

                        msg = $filter('translate')('FACEBOOK_ACCOUNT_ALREADY_IN_USE');
                        type = $_notification.getNotificationTypes().ERROR;
                    } else {
                        msg = $filter('translate')('FAILED_TO_REGISTER');
                        type = $_notification.getNotificationTypes().ERROR;
                    }

    				reject($_helper.getObjForNotification(msg, type));
    			}
    		}

            var confirmPopup = $ionicPopup.confirm({
                title: $filter('translate')('CONFIRM_EMAIL_PERMISSIONS'),
                template: $filter('translate')('CONFIRM_EMAIL_PERMISSIONS_TEXT'),
                cssClass: 'confirm-popup',
                cancelText: $filter('translate')('APP_RATE_NO'),
                okText: $filter('translate')('APP_RATE_YES')
              })
              .then(function(res) {
                var params = {
                    platform: "facebook",
                    platform_id: platform_id,
                    platform_token: platform_token,
                    platform_email: platform_email
                }
                if(res) {
                  params.mail_permission = 1;
                } 

                $_sdk.request("PUT", "/user/upgrade", params, callbacks);
                
              });

    		

    		
    	});
    }

		/*
		 * Sendet Login-Daten an den Server.
		 * @param {string} platform_id - Platform Id, von Facebook-Aufruf
		 * @param {string} platform_token - Platform Token, von Facebook-Aufruf
		 */
    function loginFacebook(platform_id, platform_token) {
    	return new Promise(function(resolve, reject){
    		var callbacks = {
    			onSuccessAck: function(successData) {
    				$log.debug("success");
    				$log.debug(successData);
    				$_sdk.setUserID(successData.user_id);
						$_sdk.setxAuthToken(successData.token);
						$_nativestorage.setItem("userID",successData.user_id);
            $_nativestorage.setItem("xAuthToken",successData.token);
            
            $_account_cache.setLoginStatus(true);
						$_account_cache.setShouldShowCreateAccountBox(false);

    				resolve(successData);
    			},
    			onFailAck: function(failData) {
    				$log.debug("fail");
    				$log.debug(failData);
                    if(failData.code == 5) {
                        msg = "email_popup";
                        type = $_notification.getNotificationTypes().ERROR;
                    } else {
        				var msg = $filter('translate')('FAILED_TO_LOGIN_VIA_FACEBOOK');
        				var type = $_notification.getNotificationTypes().ERROR;
                    }

    				var notificationObj = $_helper.getObjForNotification(msg, type);

    				notificationObj.code = failData.code;
    				reject(notificationObj);
    			},
    			onValidator: function(validateData) {
    				$log.debug("validate");
    				$log.debug(validateData);
    				var msg = $filter('translate')('FAILED_TO_LOGIN_VIA_FACEBOOK');
    				var type = $_notification.getNotificationTypes().ERROR;

    				reject($_helper.getObjForNotification(msg, type));
    			}
    		}

    		var params = {
    			platform: "facebook",
    			platform_id: platform_id,
    			platform_token: platform_token
    		}

    		$log.debug(params);

    		$_sdk.request("POST", "/user/signin", params, callbacks);
    	});
    }

	return {
    getFacebookData() {
			return getFacebookData();
    },
		registerFacebook() {
			return processFacebookRegistration();
		},
		loginFacebook() {
			return processFacebookRegistration("login");
		}
	};
});
