angular.module('tos.service', [])
.factory('$_tos', function($_sdk, $_flirt_detail_cache, $_helper, $filter, $_notification, $_tos_cache, $log) {

	/* Ruft die Nutzungsbedingungen ab */
	function getTos() {
		return new Promise(function(resolve, reject){
							var callbacks = {
									onSuccessAck: function(successData) {
										var data = $_sdk.first(successData);
										$_tos_cache.saveTOS(data);
										resolve(data);
									},
									onFailAck: function(failData) {
											var msg = $filter('translate')('REQUEST_FAILED');
											var type = $_notification.getNotificationTypes().ERROR;

											reject($_helper.getObjForNotification(msg, type));
									},
									onValidator: function(validateData) {
											var msg = $filter('translate')('FAILED_TO_LOAD_DATA');
											var type = $_notification.getNotificationTypes().ERROR;

											reject($_helper.getObjForNotification(msg, type));
									}
							}

							var params = {
								type: 'nutzungsbedingungen'
							}

							$_sdk.request("GET", "/tos", params, callbacks)
					});
	}

	/* Ruft die Datenschutzbestimmungen ab */
	function getPrivacyPolicy() {
		return new Promise(function(resolve, reject){
							var callbacks = {
									onSuccessAck: function(successData) {
										var data = $_sdk.first(successData);
										$_tos_cache.savePrivacyPolicy(data);
										resolve(data);
									},
									onFailAck: function(failData) {
										$log.debug(failData);
											var msg = $filter('translate')('REQUEST_FAILED');
											var type = $_notification.getNotificationTypes().ERROR;

											reject($_helper.getObjForNotification(msg, type));
									},
									onValidator: function(validateData) {
											var msg = $filter('translate')('FAILED_TO_LOAD_DATA');
											var type = $_notification.getNotificationTypes().ERROR;

											reject($_helper.getObjForNotification(msg, type));
									}
							}

							var params = {
								type: 'datenschutz'
							}

							$_sdk.request("GET", "/tos", params, callbacks)
					});
	}

	function updateTos(mailPermission) {
		return new Promise(function(resolve, reject){
			getTos().then(function(tos) {
				getPrivacyPolicy().then(function(privacy) {

					var callbacks = {
							onSuccessAck: function(successData) {
								resolve();
							},
							onFailAck: function(failData) {
								reject();
							},
							onValidator: function(validateData) {
								reject();
							}
					}
					
					var params = {
						tos1_version: tos.terms_of_service_id,
						tos2_version: privacy.terms_of_service_id
					}

					if(mailPermission) {
						params.mail_permission = 1;
					}

					$_sdk.request("POST", "/tos", params, callbacks);

				}, function() {
					reject();
				});
			}, function() {
				reject();
			});
		});
	}

	return {
		getTos: function() {
			return getTos();
		},
		getPrivacyPolicy: function() {
			return getPrivacyPolicy();
		},
		updateTos: function(mailPermission) {
			return updateTos(mailPermission);
		}
	};
});
